/***************************************************************************
*                             INCLUDED FILES
***************************************************************************/
#include <stdio.h>
#include <malloc.h>
#include <math.h>

/***************************************************************************
*                                CONSTANTS
***************************************************************************/
#define DEBUG 1
#define TRUE 1
#define FALSE 0
#define WORDER 5 //the order of the db-wavelet filter
#define DEPTH 4 //the max decomposition level,
#define DEPTH_ENC 4 

/*The values below must be the same in encoder and decoder!!!*/
#define DEADZONE_NUM 1//numerator and denominator for the factor which the deadzone is multiplied with
#define DEADZONE_DEN 1
#define USE_BIORTHOGONAL_WAVELETS

/***************************************************************************
*                                MACROS
***************************************************************************/
#define max(a,b) ((a)<(b)?(b):(a))
#define min(a,b) ((a)<(b)?(a):(b))
#define abs(a) ((a)<(0)?(-a):(a))


/***************************************************************************
*                            TYPE DEFINITIONS
***************************************************************************/
typedef struct {//defining the nodes in tree
	double *data,maxAbs;//J,R,D is the cost, rate and distortion respectively
	struct NODE *l,*r,*p;//l,r,p is the left/right child and parent respectively
	unsigned short  length;
	unsigned char id,R,isRLenc;

} NODE;


/***************************************************************************
*                               PROTOTYPES
***************************************************************************/
unsigned short readLength(FILE* iFile);
void createTree(NODE * node, NODE**nodes,char test);
void readData( NODE** nodes, FILE* iFile);
void invQuantDead(NODE* node, unsigned short* data);
void reconstruct(NODE **nodes,unsigned short nmbNodes);
double* wavConv(NODE *node);
double* biWavConv(NODE *node);
unsigned short * rld(unsigned short *data,unsigned short lengthIn,unsigned short lengthOut);

/***************************************************************************
*                            GLOBAL VARIABLES
***************************************************************************/
//float HR[10]={0.00333572528500155, 0.0125807519990155, -0.00624149021301171,-0.0775714938400651, -0.0322448695850295, 0.24229488706619, 0.138428145901103,-0.724308528438574, 0.603829269797473, -0.160102397974125};
//float LR[10]={0.160102397974125, 0.603829269797473, 0.724308528438574, 0.138428145901103, -0.24229488706619, -0.0322448695850295, 0.0775714938400651, -0.00624149021301171, -0.0125807519990155, 0.00333572528500155};
//the flipped versions of the filters are used for convenience
#ifdef USE_BIORTHOGONAL_WAVELETS
	float H[10] = {0, -0.037828455507264, -0.0238494650195568, 0.110624404418437, 0.377402855612831, -0.852698679008894, 0.377402855612831, 0.110624404418437, -0.0238494650195568, -0.037828455507264};
	float L[10]= {0, -0.0645388826286971, -0.0406894176091641, 0.418092273221617, 0.788485616405583, 0.418092273221617, -0.0406894176091641, -0.0645388826286971, 0, 0};
#else 
	float H[10]={-0.160102397974125,0.603829269797473,-0.724308528438574,0.138428145901103,0.24229488706619,-0.0322448695850295,-0.0775714938400651,-0.00624149021301171,0.0125807519990155,0.00333572528500155};
	float L[10]={0.00333572528500155,-0.0125807519990155,-0.00624149021301171,0.0775714938400651,-0.0322448695850295,-0.24229488706619,0.138428145901103,0.724308528438574,0.603829269797473,0.160102397974125};
#endif

/***************************************************************************
*                                FUNCTIONS
***************************************************************************/
int main(int argc,char *argv[]) {  /* BEGIN MAIN */
	FILE *iFile, *oFile;
	NODE *root, **nodes;
	unsigned short i,nmbNodes=0;

	for (i=0;i<=DEPTH;i++){nmbNodes+=pow(2,i);}
	nodes = (NODE**)calloc(nmbNodes,sizeof(NODE*));
	//open file for reading
    if ((iFile = fopen("testOut.dat","rb")) == NULL){
    	fprintf(stderr,"Cannot open file for reading\n");
	    exit(1);
  	}

  	//for (i=0;i<255;i++){printf("%d: %c ",i,(char)i);}
  	root = (NODE*)calloc(1,sizeof(NODE));
  	root->p=NULL;
  	root->id=0;
  	if (DEPTH==5&&DEPTH_ENC==4){
  		root->length=2*readLength(iFile);
  	}
  	else{
  		root->length=readLength(iFile);
  	}
	printf("root->length %d \n",root->length);
  	root->data=(double*)calloc(root->length,sizeof(double));
  	nodes[0]=root;
  	createTree((NODE*)root,(NODE**)nodes,(char)"l");
  	createTree((NODE*)root,(NODE**)nodes,(char)"r");
  	printf("ok\n");
  	readData(nodes,iFile);
  	printf("ok\n");
  	reconstruct(nodes,nmbNodes);
  	printf("\n");
  	fclose(iFile);

  	//get ready for writing data
	if ((oFile = fopen("testOutDec.dat", "wb")) == NULL)
    {
        fprintf(stderr,"Cannot open file for writing\n");
	    exit(1);
    }
	//for(i=0;i<root->length;i++){printf(" %f ",root->data[i]);}
  	fwrite(root->data,sizeof(double),root->length,oFile);
  	fclose(oFile);


  	/*if ((oFile = fopen("testOutDecN1.dat", "wb")) == NULL)
    {
        fprintf(stderr,"Cannot open file for writing\n");
	    exit(1);
    }
	//for(i=0;i<root->length;i++){printf(" %f ",root->data[i]);}
  	fwrite(nodes[1]->data,sizeof(double),nodes[1]->length,oFile);
  	fclose(oFile);*/
	return 0;
}

/***************************************************************************
* This function reads the length of the signal after cut. 
*---------------------------------------------------------------------------
* Input:   iFile   - input stream
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-04-21
***************************************************************************/
unsigned short readLength(FILE* iFile){
	int c;
	unsigned short length;
	
	c=fgetc(iFile);
	//reading dynamic range(maxAbs)
	//printf("c %d\n",c);
	length=(unsigned short)c;
	length<<=8;
	if ((c=fgetc(iFile))==EOF){
		fprintf(stderr,"Reached EOF when reading maxAbs\n");
		exit(0);;
	}
	length=length|((unsigned short) c);
	return length;
}

/***************************************************************************
* This function creates a binary tree where each node represents a wavelet
* packet. Initially all the node's datapoints are set to zero. Also the pointer
* to the pointers of nodes is initialized.
*---------------------------------------------------------------------------
* Input:   node   - pointer to a struct representing a node
* 			nodes - pointer to pointer of nodes
* 		   test   - check if the new node is the left or right child
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-02-26
***************************************************************************/
void createTree(NODE * node, NODE**nodes,char test){
	NODE * newNode;
	unsigned short lh=10,ldata=(*node).length;
	unsigned short ly=(short)(lh+ldata-1)/2; //declaring length of data-array
	//checking if the new node is left or right child of parent
	if (test==(char)"l"){
		(*node).l=calloc(1,sizeof(NODE));
		newNode=(NODE*)(*node).l;//the link between child an parent
		(*newNode).id=2*(*node).id+1; //the child's node id
		//printf(" \n Det ble lavpass");
		#ifdef USE_BIORTHOGONAL_WAVELETS
			ly=(short)(ldata+1)/2;
		#else
			ly=(short)(lh+ldata-1)/2;
		#endif
	}
	if (test==(char)"r"){
		(*node).r=calloc(1,sizeof(NODE));
		newNode=(NODE*)(*node).r;//the link between child an parent
		(*newNode).id=2*(*node).id+2; //the child's node id
		//printf(" \n Det ble hypass");
		#ifdef USE_BIORTHOGONAL_WAVELETS
			ly=ldata-(short)(ldata+1)/2;;
		#else
			ly=(short)(lh+ldata-1)/2;
		#endif
	}
	nodes[newNode->id]=newNode;
	newNode->p=node;//the link between child an parent
	(*newNode).length=ly; //the length of the child's data array
	newNode->data=(double*)calloc(ly,sizeof(double));
	newNode->R=0;//initially set the rate to zero
	if ((*newNode).id<pow(2,DEPTH)-1){
		createTree(newNode,nodes,(char)"l");
		createTree(newNode,nodes,(char)"r");
	}
	else{
		(*newNode).l=NULL;
		(*newNode).r=NULL;
		//printf("id %d\n",newNode->id);
	}


}


/***************************************************************************
* This function reads the stream pointed to by iFile and writes it to the correct
* nodes in the tree.
*---------------------------------------------------------------------------
* Input:	root - pointer to the root node in our tree
* 			nodes - pointer to pointer of nodes
* 			iFile  - pointer to the instream which we will read our data from
*
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-02-26
***************************************************************************/
void readData(NODE** nodes, FILE* iFile){
	unsigned char nextLeaf,tmpID,bitbuffer,cc,tmpR;
	unsigned short tmpMA,tmpData,i,j,tmpLength,mask;
	NODE *node;
	int c;
	node = (NODE*)calloc(1,sizeof(NODE));
	tmpID=0;
	nextLeaf= FALSE;
	bitbuffer=0;
	tmpR=0;
	tmpLength=0;

	c=fgetc(iFile);
	mask=1<<7;
	while (c != EOF){
		//reading dynamic range(maxAbs)
		//printf("c %d\n",c);
		tmpMA=(unsigned short)c;
		tmpMA<<=8;
		if ((c=fgetc(iFile))==EOF){
			fprintf(stderr,"Reached EOF when reading maxAbs\n");
			break;
		}
		printf("c %d\n",c);
		tmpMA=((unsigned char)c)|tmpMA;
		//reading id
		if ((c=fgetc(iFile))==EOF){
			fprintf(stderr,"Reached EOF when reading ID\n");
			break;
		}
		cc=(unsigned char)c;
		for (i=0;i<6;i++){
			tmpID=(((cc&mask)>>7)|tmpID);
			bitbuffer++;
			cc<<=1;
			if(i<5){
				tmpID<<=1;
			}
		}
		//finding the node with the corresponding id since the encoder uses a smaller tree(4)
		if (DEPTH==5&&DEPTH_ENC==4){
			i=0;
			while((short)pow(2,i)<=(tmpID+1)){
				i++;
			}
			printf(" tmpID fr %d i %d\n",tmpID,i);
			tmpID+=(short)pow(2,i-1);
		}
		//finding the node with the corresponding id since the encoder uses a smaller tree(3)
		/*i=0;
		while((short)pow(2,i)<=(tmpID+1)){
			i++;
		}
		tmpID+=(short)pow(2,i-1);*/
		//printf("c rett etter  %d tmpID %d\n",c,tmpID);

		node = nodes[tmpID];
		node->maxAbs=(float)tmpMA;
		printf("ID %d length %d maxAbs %f",node->id,node->length,node->maxAbs);


		//reading rate
		for (i=0;i<4;i++){
			if (bitbuffer==8){
				bitbuffer=0;
				if ((c=fgetc(iFile))==EOF){
					fprintf(stderr,"Reached EOF when reading rate\n");
					break;
				}
				cc=(unsigned char)c;
			}
			tmpR=(((cc&mask)>>7)|tmpR);
			bitbuffer++;
			cc<<=1;
			if(i<3){
				tmpR<<=1;
			}
		}
		node->R=tmpR;
		printf(" rate %d",node->R);

		//reading isRLenc
		if (bitbuffer==8){
			bitbuffer=0;
			if ((c=fgetc(iFile))==EOF){
				fprintf(stderr,"Reached EOF when reading rate\n");
				break;
			}
			cc=(unsigned char)c;
		}
		node->isRLenc=(cc&mask)>>7;
		bitbuffer++;
		cc<<=1;
		printf(" isRLenc %d\n",node->isRLenc);

		//readig length of rle data if used
		if (node->isRLenc==TRUE){
			//printf(" c hex %x bitbuffer %d\n",c,bitbuffer);
			for (i=0;i<10;i++){
				//printf(" i %d c hex %x bitbuffer %d tmpLength %x %d\n",i,c,bitbuffer,tmpLength,tmpLength);
				if (bitbuffer==8){
					bitbuffer=0;

					if ((c=fgetc(iFile))==EOF){
						fprintf(stderr,"Reached EOF when reading rate\n");
						break;
					}
					cc=(unsigned char)c;

				}
				tmpLength=(((cc&mask)>>7)|tmpLength);
				bitbuffer++;
				cc<<=1;
				if(i<9){
					tmpLength<<=1;
				}
			}
			printf("RLE and tmpLength %d\n",tmpLength);
		}
		else{
			tmpLength=node->length;
		}

		//read the incoming data into an arry
		if (node->R>0){
			printf("Writing out data before rld: node->R %d \n",node->R);
			tmpData=0;
			unsigned short *data=(unsigned short *)calloc(tmpLength,sizeof(unsigned short));
			for(i=0;i<tmpLength;i++)
			{
				if(node->R<4)
				{
					tmpR=4;
				}
				else
				{
					tmpR=node->R;
				}
				for (j=0;j<tmpR;j++){
					if (bitbuffer==8){
						bitbuffer=0;
						if ((c=fgetc(iFile))==EOF){
							fprintf(stderr,"Reached EOF when reading data\n");
							exit(0);break;
						}
						cc=(unsigned char)c;
					}
					tmpData=(((cc&mask)>>7)|tmpData);
					bitbuffer++;
					cc<<=1;
					if (j<(tmpR-1)){
						tmpData<<=1;
					}
					//if(node->id==17 && i<10)printf("j %d tmpData %d c %x bitbuffer %d\n ",j,tmpData,c,bitbuffer);
				}
				data[i]=tmpData;
				//printf("%d ",tmpData);
				tmpData=0;

			}
			if (node->isRLenc==TRUE){
				data=(unsigned short*)rld(data,tmpLength,node->length);
			}
			/*printf("Data after rld:\n");
			for (i=0;i<node->length;i++){
				printf("%d ", data[i]);
			}*/

			invQuantDead(node,data);
			
			//printf("Data after rld:\n");
			/*for (i=0;i<node->length;i++){
				printf("%f ", node->data[i]);
			}*/
			free(data);
		}
		c=fgetc(iFile);
		bitbuffer=0;
		tmpID=0;
		tmpR=0;
		tmpMA=0;
		tmpLength=0;
		printf("Finished with %d rate %d\n",node->id,node->R);
	}

}


/***************************************************************************
* This function makes the inverse quantization for a uniform quantizer with deadzone and maps the quantization indicies
* to real values.
*---------------------------------------------------------------------------
* Input:	node - pointer to a struct representing a node
* 			data  - pointer to an array representing the quantization indicies
* 					for this node.
*
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-04-30
***************************************************************************/
void invQuantDead(NODE* node, unsigned short* data){

	long stepLengthOver,stepLengthUnder,stepLength,tmp;
    short repLevels,i,zeroRepresentation;
    repLevels=(short)pow(2,node->R);
    
    zeroRepresentation=(repLevels>>1)-1;
    stepLength = 2*node->maxAbs/repLevels;
    tmp=((stepLength*DEADZONE_NUM)>>1)/DEADZONE_DEN;
    stepLengthOver=(node->maxAbs-tmp)/(repLevels>>1);
    stepLengthUnder=(node->maxAbs-tmp)/((repLevels>>1)-1);

	node->data=(double*)calloc(node->length,sizeof(double));
	printf("writing reconstructed values id %d:\n",node->id );
	for (i=0;i<node->length;i++){
		if(data[i]==zeroRepresentation){
    		node->data[i]=0;
    	}
        else if(data[i]>zeroRepresentation){
            node->data[i]=(double)(stepLengthOver>>1)+tmp+(data[i]-zeroRepresentation-1)*stepLengthOver;
        }
        else{
            node->data[i]=(double)(-stepLengthUnder>>1)-tmp+(data[i]-zeroRepresentation+1)*stepLengthUnder;
        }
        printf(" %.1f ", node->data[i]);
	}
	
}

/***************************************************************************
* Funcion for wavelet packet reconstruction. The reconstructed data is put
* in the root node's data-field.
*---------------------------------------------------------------------------
* Input:	nodes - pointer to pointer of nodes
* 			nmbNodes - the total number of nodes in the tree
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-02-28
***************************************************************************/
void reconstruct(NODE **nodes,unsigned short nmbNodes){
	int i,j,length;
	double *d1,*d2;
	NODE *n1,*n2;
	i=nmbNodes-1;
	printf(" reconstructing %d\n", nmbNodes);
	while(i>0){
		n1=nodes[i-1];
		n2=nodes[i];
		length=((NODE*)(n1->p))->length;
		if((n1->R!=0)&&(n2->R!=0)){
			//printf(" both %d and %d have values %d\n",n1->id, n2->id,i);

			#ifdef USE_BIORTHOGONAL_WAVELETS
				d1=biWavConv(n1);
				d2=biWavConv(n2);
			#else
				d1=wavConv(n1);
				d2=wavConv(n2);
			#endif
			for (j=0;j<length;j++){
				((NODE*)(n1->p))->data[j]=d1[j]+d2[j];
			}
			if (n1->id==13){
				FILE *f;
				f=fopen("341.dat","wb");
				fwrite(n1->data,sizeof(double),n1->length,f);
				fwrite(n2->data,sizeof(double),n2->length,f);
				fwrite(((NODE*)(n1->p))->data,sizeof(double),((NODE*)(n1->p))->length,f);
				fclose(f);
			}
			((NODE*)n1->p)->R=1;//we must set the parent's rate to a value above 0

		}
		else if ((n1->R==0)&&(n2->R!=0)){
			//printf(" %d has values %d\n", n2->id,i);
			#ifdef USE_BIORTHOGONAL_WAVELETS
				d2=biWavConv(n2);
			#else
				d2=wavConv(n2);
			#endif
			for (j=0;j<length;j++){
				((NODE*)(n1->p))->data[j]=d2[j];
			}
			((NODE*)n1->p)->R=1;//we must set the parent's rate to a value above 0
		}
		else if ((n1->R!=0)&&(n2->R==0)){
			//printf(" %d has values %d\n", n1->id,i);
			#ifdef USE_BIORTHOGONAL_WAVELETS
				d1=biWavConv(n1);
			#else
				d1=wavConv(n1);
			#endif
			for (j=0;j<length;j++){
				((NODE*)(n1->p))->data[j]=d1[j];
			}
			((NODE*)n1->p)->R=1;//we must set the parent's rate to a value above 0

		}
		else{
			//printf(" neither %d nor %d has values %d\n",n1->id, n2->id,i);
		}
		i-=2;
	}
	printf(" reconstructing finished\n");
}

/***************************************************************************
* Funcion which takes care of the actual convolution and interpolation.
*---------------------------------------------------------------------------
* Input:	nodes - pointer to pointer of nodes
* Output: 	y	 - 	pointer to array of filtered data
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-02-28
***************************************************************************/
double* wavConv(NODE *node){
	float *h;
	double *y,*intData;
	int i,j,in,lp,lIntData,count,startIntData,hi,lh;
	//initializing indicies/lengths for use in the convolution
	startIntData=0;
	lh=2*WORDER;
	in=2*(WORDER-1)+1;
	hi=-lh+in;
	lIntData=2*(node->length);
	lp=((NODE*)node->p)->length;
	//creating interpolated data
	intData =(double*)calloc(lIntData,sizeof(double));

	for (i=0;i<node->length;i++){
		intData[2*i]=node->data[i];
		if (node->id==55||node->id==27||node->id==56){
			printf(" %f",node->data[i]);
		}
	}

	//checking which filter to use
	if ((node->id%2)==0){
		printf("Highpass %d\n",node->id);
		h=H;
	}
	else{
		printf("Lowpass %d\n",node->id);
		h=L;
	}

	y= (double*)calloc(lp,sizeof(double));
	for (i=0;i<lp;i++){
		//printf("Inside wavConv\n");
		count=0;
		for (j=max(startIntData,hi);j<=min(hi+lh-1,lIntData-1);j++){
			if((i==0)&&(node->id==14)){
				//printf(" %d %d %d %f %f\n",max(startIntData,hi),min(hi+lh-1,lIntData-1),j,intData[j],h[lh-in+count]);
			}
			y[i]+=intData[j]*h[lh-in+count];
			count++;
		}
		if (in<lh){
			in++;
		}
		hi++;
	}
	free(intData);
	return y;
}

/***************************************************************************
* Funcion which takes care of the actual convolution and interpolation.
*---------------------------------------------------------------------------
* Input:	nodes - pointer to pointer of nodes
* Output: 	y	 - 	pointer to array of filtered data
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-04-29
***************************************************************************/
double* biWavConv(NODE *node){
	float *h;
	double *y,*intData;
	int i,j,lp,lIntData,hi,lh,count;
	//initializing indicies/lengths for use in the convolution
	lh=2*WORDER;
	hi=0;
	lIntData=2*(node->length);
	lp=((NODE*)node->p)->length;
	//creating interpolated data
	intData =(double*)calloc(lIntData+12,sizeof(double));

	for (i=0;i<node->length;i++){
		intData[6+2*i]=node->data[i];
		if (node->id==7||node->id==27||node->id==56){
			//printf(" %f",node->data[i]);
		}
	}
	//checking which filter to use and symmetric extension
	if ((node->id%2)==0){
		printf("Highpass %d\n",node->id);
		h=H;
		if(lp%2==0){
			for(j=0;j<3;j++){
				intData[2*j]=node->data[2-j];//(2,1) if even parent
				intData[lIntData+6+2*j]=node->data[node->length-2-j];   
			}				
		}
		else{
			for(j=0;j<3;j++){
				intData[2*j]=node->data[2-j];//(2,2) if odd parent
				intData[lIntData+6+2*j]=node->data[node->length-1-j];
			}
		}
	}
	else{
		printf("Lowpass %d\n",node->id);
		h=L;
		if(lp%2==0){
			for(j=0;j<3;j++){
				intData[2*j]=node->data[3-j];//(1,2) if even parent
				intData[lIntData+6+2*j]=node->data[node->length-1-j];
			}
		}
		else{
			for(j=0;j<3;j++){
				intData[2*j]=node->data[3-j];//(1,1) if odd parent
				intData[lIntData+6+2*j]=node->data[node->length-2-j];
			}
		}
		intData+=2;//must remove first element for lowpass in order to keep symmetry
	}
	if (node->id==9){
		FILE *f;
		f=fopen("9.dat","wb");
		fwrite(intData,sizeof(double),lIntData+10,f);
		fclose(f);
		//printf("lIntData %d id%d\n",lIntData, node->id);
	}
	if (node->id==10){
		FILE *f;
		f=fopen("10.dat","wb");
		fwrite(intData,sizeof(double),lIntData+12,f);
		fclose(f);
		//printf("lIntData %d id%d\n",lIntData, node->id);
	}

	y= (double*)calloc(lp,sizeof(double));

	for (i=0;i<lp;i++){
		//printf("Inside wavConv\n");
		count=0;
		for (j=hi;j<hi+lh;j++){
			if((i==0)&&(node->id==14)){
				//printf(" %d %d %d %f %f\n",max(startIntData,hi),min(hi+lh-1,lIntData-1),j,intData[j],h[lh-in+count]);
			}
			y[i]+=intData[j]*h[count];
			count++;
		}
		
		hi++;
	}
	free(intData);
	return y;
}

/***************************************************************************
* Function for run length decoding
*---------------------------------------------------------------------------
* Input:   data   -	pointer to a struct representing a node in the tree
* 		   lengthIn - length of data-vector
* 		   lengthOut - length of vector after run-length decoding
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-03-20
***************************************************************************/
unsigned short * rld(unsigned short *data,unsigned short lengthIn,unsigned short lengthOut){
	unsigned short *y,past,yi=1,test=FALSE,i,j;
	past=data[0];
	y=(unsigned short *)calloc(lengthOut,sizeof(unsigned short));
	y[0]=past;
	//printf("\n");
	for(i=1;i<lengthIn;i++)
	{
		if (test==TRUE){
			for(j=0;j<data[i];j++){
				y[yi+j]=past;
			}
			//printf("[1]i %d d %d p %d ",i, data[i],past);
			yi+=data[i];
			past=1<<14;
			test = FALSE;
		}
		else if (data[i]==past){
			//printf("[2]i %d d %d p %d ",i, data[i],past);
			test = TRUE;
			y[yi]=data[i];
			yi++;
		}
		else{
			//printf("[3]i %d d %d p %d ",i, data[i],past);
			y[yi]=data[i];
			yi++;
			past=data[i];
		}
	}
	//printf("Data after rld:\n");
	for (i=0;i<lengthOut;i++){
		//printf("%d ", y[i]);
	}
	return y;
}



