/***************************************************************************
*                             INCLUDED FILES
***************************************************************************/
#include "xparameters.h"
#include "xuartlite_l.h"
#include "xtmrctr_l.h"

/***************************************************************************
*                                CONSTANTS
***************************************************************************/
//#define DEBUG
#define TRUE 1
#define FALSE 0 
#define RBUDGET 3000
#define OVERHEAD 26
#define FILTER_LENGTH 10
#define BI_LP_FILTERLENGTH 9
#define BI_HP_FILTERLENGTH 7
#define INITIAL_LENGTH_OF_TRACES 2048
/***************************************************************************
*       DEFINES which decides the properties of the decoder. 
***************************************************************************/
#define USE_ESTIMATED_DISTORTION //estimated or real...
#define BEST_WP_BASES //use this if BEST_WP_BASES algorithm should be used, otherwise a simple bit-allocation scheme is used
#define NUM_VAR 2 //numerator and denominator for the factor which the variance is divided on
#define DEN_VAR 1
#define DEPTH 4 //the decomposition level


/*The values below must be the same in encoder and decoder!!!*/
#define DEADZONE_NUM 7//numerator and denominator for the factor which the deadzone is multiplied with
#define DEADZONE_DEN 5
//#define USE_BIORTHOGONAL_WAVELETS

/***************************************************************************
*                                RAM-ADRESSES
***************************************************************************/
#define TREE_BASEADDR 0x20180000 //4096 bytes
#define TREE_HIGHADDR 0x20180FFF
#define TRACE_BASEADDR 0x20181000 //32768 bytes
#define TRACE_HIGHADDR 0x20188FFF
#define LEAF_BASEADDR 0x20189000 //8192 bytes
#define LEAF_HIGHADDR 0x2018BFFF
#define RLIST_BASEADDR 0x2018C000 //4096 bytes
#define RLIST_HIGHADDR 0x2018CFFF
#define DLIST_BASEADDR 0x2018D000  //4096 bytes
#define DLIST_HIGHADDR 0x2018DFFF
#define Q_DATA_BASEADDR 0x2018E000 //4096 bytes
#define Q_DATA_HIGHADDR 0x2018EFFF
#define RLE_DATA_BASEADDR 0x2018F000 //4096 bytes
#define RLE_DATA_HIGHADDR 0x2018FFFF 
#define LEAFNODES_BASEADDR 0x20190000 //200 bytes
#define LEAFNODES_HIGHADDR 0x201901FF 

	

/***************************************************************************
*                                MACROS
***************************************************************************/
#define max(a,b) ((a)<(b)?(b):(a))
#define min(a,b) ((a)<(b)?(a):(b))
#define abs(a) ((a)<(0)?(-a):(a))


/***************************************************************************
*                            TYPE DEFINITIONS
***************************************************************************/

typedef struct {//defining the nodes in tree
    struct NODE *l,*r;//l,r is the left/right child respectively
    short id, length,split,nmbOfQ;
    long *data, maxAbs;
    unsigned char *R_list;
    unsigned long *D_list,totR,J,R,D,variance;
} NODE;
typedef struct {//defining the final leaves
    unsigned short maxAbs,length,*data,lengthRl,*rlData;
    unsigned char id,R,isRLenc;
} LEAF;


/***************************************************************************
*                               PROTOTYPES
***************************************************************************/
void biPreprocess(NODE *node,long *data);
void preprocess(NODE *node,long *data);
void wavdec(NODE *inNode,  char c);
void biWavdec(NODE *inNode, char test);
void initRD(NODE *node);
void setCost(NODE *node, long *lambda);
void findSubTree(NODE* node, short nmb);
void createLeaves(NODE* node);
void unifQuant(NODE* node, LEAF * leaf);
void unifQuantDead(NODE* node, LEAF * leaf);
void writeLeaf(LEAF* leaf);
void rle(LEAF* leaf);
void writeLength(unsigned short length);
void writeLeaves(NODE** nodes);
void allocateBits(NODE** nodes);
void wavdecRE(NODE *inNode,  char c,NODE** nodes);
void biWavdecRE(NODE *inNode, char test,NODE** nodes);
void findBestSubTree(NODE* root, short nmb);


/***************************************************************************
*                            GLOBAL VARIABLES
***************************************************************************/
//const float HD= [-0.0645388826286971 0.0406894176091641 0.418092273221617 -0.788485616405583 0.418092273221617 0.0406894176091641 -0.0645388826286971];
//const float LD= [0.037828455507264 -0.0238494650195568 -0.110624404418437 0.377402855612831 0.852698679008894 0.377402855612831 -0.110624404418437 -0.0238494650195568 0.037828455507264];
#ifdef USE_BIORTHOGONAL_WAVELETS
	long H[7]= {-645, 406, 4180, -7884, 4180, 406, -645};
	long L[9]= {378, -238, -1106, 3774, 8526, 3774, -1106, -238, 378};

#else
	long H[10]={33, 125, -62,-775, -322, 2422, 1384,-7243, 6038, -1601};
	long L[10]={1601, 6038, 7243, 1384, -2422, -322, 775, -62, -125, 33};
#endif
//pointers to keep track of the static adress-lists
unsigned long  tracePtr,  D_listPtr,R_listPtr,nodePtr, leafPtr,qdataPtr,rleDataPtr,leafNodesPtr;



/***************************************************************************
*                                FUNCTIONS
***************************************************************************/

int main (void) {


	#ifdef DEBUG 
		print("-- Entering main() --\r\n");
	#endif
	print("-- Entering main() --\r\n");
	short  i=0,testLambda=FALSE,nbNodes=0,cc,j,index;
	unsigned short lengthOfSignal;
	long lambdaUL=10000, RUL,RLL,lambdaLL=0,lambda,DUL,DLL;
    NODE *root;
	NODE **leafNodes;
	long tmpMaxAbs,*inp,tmpMax;
    unsigned long tic,tic2,tic3,tic4,tic5;
    unsigned char c;
    tracePtr=(unsigned long)TRACE_BASEADDR;
    nodePtr= (unsigned long)TREE_BASEADDR;
    leafPtr = (unsigned long)LEAF_BASEADDR;
    qdataPtr=(unsigned long)Q_DATA_BASEADDR;
    rleDataPtr=(unsigned long)RLE_DATA_BASEADDR;
	leafNodesPtr=(unsigned long)LEAFNODES_BASEADDR;

    inp=(long*)TRACE_HIGHADDR-INITIAL_LENGTH_OF_TRACES;
	R_listPtr=(unsigned long)RLIST_BASEADDR;
	D_listPtr=(unsigned long)DLIST_BASEADDR;
	nbNodes=1;
    for (i=1;i<=DEPTH;i++){nbNodes+=(2<<(i-1));}
	leafNodes=LEAFNODES_BASEADDR;
	//reading data from com-port
	i=0;
	while (i<INITIAL_LENGTH_OF_TRACES){
		c = XUartLite_RecvByte((unsigned long) XPAR_RS232_BASEADDR);
		cc=(short)c;
		cc=cc<<8;
		c = XUartLite_RecvByte((unsigned long) XPAR_RS232_BASEADDR);
		cc=((short)c)|cc;
		inp[i]=(long)cc;
		i++;
	}
    //tmpMaxAbs=0;
	//xil_printf("starting...%d %d\r\n",inp[10],inp[100]);
	
	//starting timer which counts clock cycles
	XTmrCtr_mEnable(XPAR_OPB_TIMER_0_BASEADDR, 1);
	tic = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1);
	tic5 = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1);	
	tmpMax=-33000;
    for (i=0;i<INITIAL_LENGTH_OF_TRACES;i++){
        if (inp[i]>tmpMax){
			tmpMax=inp[i];
            index=i;
        }
    }
	if (index%2==1){
		index++;
	}
	index-=200;
	if (index<0)
	{
		index=0;
	}

	root=(NODE*) nodePtr;
    nodePtr+=sizeof(NODE);//increase value of pointer
    (*root).id=0;
    root->length=INITIAL_LENGTH_OF_TRACES-index;
	#ifdef DEBUG 
		xil_printf("length before preprocessing %d index %d\r\n", (*root).length,index); 
		for (i=0;i<10;i++){xil_printf(" %ld", inp[i+index]);}
	#endif
	//preprocess((NODE*) root,(float*) inp);
    /*********************
     * testing
     * *****************/
    root->maxAbs=32000;//just putting a value
    root->data=(long*)tracePtr;
    tracePtr+=sizeof(long)*(((root->length+1)>>1)+((FILTER_LENGTH-1)<<1));
    if(DEPTH <5){
        /*#ifdef USE_BIORTHOGONAL_WAVELETS
        	biPreprocess(root,input);
        #else
        	preprocess(root,input);
        #endif*/
        
		root->length=(root->length+1)>>1;
        if(root->length%2==1){
        	root->length--;
        }
		root->data+=FILTER_LENGTH-1;
        for (i=0;i<root->length;i++){
            root->data[i]=(inp[index+(i<<1)]>>1)+(inp[index+(i<<1)+1]>>1);
            //xil_printf(" %d", root->data[i+FILTER_LENGTH-1]);
        }
		root->data-=(FILTER_LENGTH-1);
        if(DEPTH ==3){
            #ifdef USE_BIORTHOGONAL_WAVELETS
	        	biPreprocess(root,root->data);
	        #else
	        	preprocess(root,root->data);
	        #endif
        }
    }
	
	#ifdef DEBUG 
		xil_printf("length after preprocessing %d\r\n", (*root).length); 
		for (i=0;i<10;i++){xil_printf(" %ld", root->data[i+FILTER_LENGTH-1]);}
	#endif
	#ifdef USE_BIORTHOGONAL_WAVELETS
		for (i=0;i<4;i++)
	    {
	        root->data[FILTER_LENGTH-2-i]=root->data[i+FILTER_LENGTH];
			#ifdef DEBUG 
				xil_printf(" %ld", root->data[FILTER_LENGTH-2-i]);
			#endif
			root->data[root->length+FILTER_LENGTH-1+i]=root->data[root->length+FILTER_LENGTH-3-i];
			
	    }
		root->data+=5;
		#ifdef DEBUG 
			xil_printf("\r\n"); for (i=0;i<10;i++){xil_printf(" %ld", root->data[i]);}xil_printf("\r\n");
		#endif
    #else
    	for (i=0;i<FILTER_LENGTH-1;i++)
		{
			root->data[FILTER_LENGTH-2-i]=root->data[i+FILTER_LENGTH-1];
			root->data[root->length+FILTER_LENGTH-1+i]=root->data[root->length+FILTER_LENGTH-2-i];
			#ifdef DEBUG 
				xil_printf(" %ld", root->data[FILTER_LENGTH-2-i]);
			#endif
		}
		#ifdef DEBUG 
			xil_printf("\r\n"); for (i=0;i<10;i++){xil_printf(" %ld", root->data[i+5]);}xil_printf("\r\n");
		#endif
	#endif 
	tic5 = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1)-tic5;	
	lengthOfSignal=root->length;
	/*********************
     * end testing
     * *****************/
	
	#ifdef DEBUG 
		xil_printf("root->length %d\r\n", (short)root->length);
	#endif
		
	#ifdef BEST_WP_BASES
	    /*running wavedec-function for left and right child, i.e. filtering with
	    * high-pass and low-pass and decimating with a factor two. The tree-structure is created*/
		tic2 = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1);
	    #ifdef USE_BIORTHOGONAL_WAVELETS
	    	biWavdec(root,(char)"l");
	    	biWavdec(root,(char)"r");	
	    #else
	    	wavdec(root,(char)"l");
	    	wavdec(root,(char)"r");
	    #endif
		tic2 = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1)-tic2;
	    //creating the R-D points for all the nodes in the tree
		tic3 = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1);
	    initRD(root);
		tic3 = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1)-tic3;
	    findBestSubTree(root,nbNodes);
    #else
    	/*running wavedec-function for left and right child, i.e. filtering with
	    * high-pass and low-pass and decimating with a factor two. The tree-structure is created*/
		tic2 = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1);
	    #ifdef USE_BIORTHOGONAL_WAVELETS
	    	biWavdecRE(root,(char)"l",leafNodes);
	    	biWavdecRE(root,(char)"r",leafNodes);	
	    #else
	    	wavdecRE(root,(char)"l",leafNodes);
	    	wavdecRE(root,(char)"r",leafNodes);
	    #endif
	    tic2 = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1)-tic2;
	    //creating the R-D points for all the nodes in the tree
		tic3 = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1);
	    allocateBits(leafNodes);
		tic3 = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1)-tic3;
	#endif
    
     //for (i=0;i<2;i++){xil_printf("Data %d %x %ld\r\n", (short) (root->data)[103+i],&(root->data)[103+i],&(root->data)[1003+i]);}
	 //xil_printf("J %x R %x D %x maxAbs %x\r\n", (Xuint32)(&(root->J)),(Xuint32)(&(root->R)), (Xuint32)(&(root->D)), (Xuint32) (&(root->maxAbs)));
	 //xil_printf("Root %x data %x l %x id %x\r\n", (Xuint32) root,(Xuint32) root->data, (Xuint32)(&(root->length)), (Xuint32) (&(root->id)));  
	 //xil_printf("nodePtr  %x %d D_listPtr %x length %d\r\n", nodePtr,nodePtr,D_listPtr,root->length);
    
	
	
	
	
	
	//write header (length of signal)
    writeLength(lengthOfSignal);
 	#ifdef BEST_WP_BASES
		tic4 = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1);
	    createLeaves(root);
		tic4 = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1)-tic4;
	#else
		tic4 = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1);
		writeLeaves(leafNodes);
		tic4 = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1)-tic4;
	#endif
	
	tic = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1)-tic;	
	xil_printf("-- Exiting main() --Tot %d Filt %d Init/alloc %d LeafRLE %d Pre: %d\r\n", tic,tic2,tic3,tic4,tic5);
   
   return 0;
}

/***************************************************************************
* Function for preprocessing the signal. The function basically decimates
* with two and only keeps the lowpass samples. This is because the signal is
* oversampled and there is little information in the upper half of the frequency
* band                
*---------------------------------------------------------------------------
* Input:   node   - pointer to a struct representing a node, with its 
*                                           belonging data-sequence etc.  
*          data  - original data sequence                       
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-03-10 
***************************************************************************/
void preprocess(NODE * node, long *data){
		
        short k,ldata=node->length, lh=FILTER_LENGTH;
		short ly=(short)((lh+ldata-1)>>1); //declaring length of data-array
		short  ih=1;
		long *y; //pointer to array of filtered/decimated data
		long *h; //pointer to array of filter-coeffisents
		long tmpMaxAbs=-1;
        
		h=L;
        y=(long*)tracePtr+FILTER_LENGTH-1;
        tracePtr+=sizeof(long)*(ly+2*(FILTER_LENGTH-1));
        for (k=0;k<ly;k++){
            int teller=0, p;
            y[k]=0;
            for (p=ih;p<ih+lh;p++){
                y[k]+=data[p]*h[teller];
                teller = teller+1;
            }
			y[k]/=10000;
			//xil_printf(" %d",y[k+FILTER_LENGTH-1]);
            if (abs(y[k])>tmpMaxAbs){
                    tmpMaxAbs=abs(y[k]);
                    //xil_printf("storre %d abs %d tmp %d\r\n", y[k+FILTER_LENGTH-1],abs(y[k+FILTER_LENGTH-1]),tmpMaxAbs);
            }
            ih=ih+2;
            
        }
        y-=(FILTER_LENGTH-1);
		for (k=0;k<FILTER_LENGTH-1;k++)
		{
			y[FILTER_LENGTH-2-k]=y[k+FILTER_LENGTH-1];
			y[ly+FILTER_LENGTH-1+k]=y[ly+FILTER_LENGTH-2-k];
		}
		node->data=y;
        node->length=ly;
		node->maxAbs=tmpMaxAbs;
        //xil_printf("Preprocessing %d %d\r\n", node->length, node->maxAbs);
}

/***************************************************************************
* Function for preprocessing the signal with biorthogonal wavelets. 
* The function basically decimates
* with two and only keeps the lowpass samples. This is because the signal is
* oversampled and there is little information in the upper half of the frequency
* band
*---------------------------------------------------------------------------
* Input:   node   - pointer to a struct representing a node, with its
* 					  belonging data-sequence etc.
* 			data  - original data sequence
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-03-10
***************************************************************************/
void biPreprocess(NODE * node, long *data){
    short k,ldata=node->length, lh=BI_LP_FILTERLENGTH;
    short ly=(short)(ldata+1)/2; //declaring length of data-array
    short  ih=0;
    long *y; //pointer to array of filtered/decimated data
    long *h; //pointer to array of filter-coeffisents
    long tmpMaxAbs=-1;
	y=(long*)tracePtr+FILTER_LENGTH-1;
    tracePtr+=sizeof(long)*(ly+2*(FILTER_LENGTH-1));
    h=L;
	
    for (k=0;k<ly;k++){
        int teller=0, p;
        for (p=ih;p<ih+lh;p++){
        	y[k]=0;
            //printf(" \n Og p er: %d", p);
            y[k]=y[k]+data[p]*h[teller];
            //printf(" \n Og data[p] er: %f og h[lh-in+teller] er: %f ", (*inNode).data[p],h[lh-in+teller]);
            teller = teller+1;
        }
        y[k]/=10000;
        if (abs(y[k])>tmpMaxAbs){
            tmpMaxAbs=abs(y[k]);
        }
        //if(k>65&&k<100)printf(" %ld", (long)y[k]);

        ih=ih+2;
    }
    y-=(FILTER_LENGTH-1);
    for (k=0;k<4;k++)
    {
        y[FILTER_LENGTH-2-k]=y[k+FILTER_LENGTH];
		y[ly+FILTER_LENGTH-1+k]=y[ly+FILTER_LENGTH-3-k];
    }
    y+=5;
    node->data=y;
    node->length=ly;
    node->maxAbs=tmpMaxAbs;
}

/***************************************************************************
* Function for 1D WP analysis with decimation. The function recursively calls
* itself untill the desired depth is reach. A tree-structure which links every
* node together is created as well.                
*---------------------------------------------------------------------------
* Input:   inNode   - pointer to a struct representing a node, with its 
*                                           belonging data-sequence etc.                         
*          test            - this value decides if the LP or HP filter is used
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-02-16 
***************************************************************************/
void wavdec(NODE *inNode, char test){
                                
    NODE *newNode;
    short k,ldata=(*inNode).length, lh=FILTER_LENGTH;
	short ly=(short)((lh+ldata-1)>>1); //declaring length of data-array
	short ih=1,lag;
	long *y; //pointer to array of filtered/decimated data
	long *h; //pointer to array of filter-coeffisents
	long tmpMaxAbs=0;
                    
    //checking if the new node is left or right child of parent
    if (test==(char)"l"){
    	(*inNode).l = (NODE*)nodePtr;
    	nodePtr+=sizeof(NODE);
        newNode=(NODE*)(*inNode).l;//the link between child an parent
        (*newNode).id=((*inNode).id<<1)+1; //the child's node id
        //xil_printf("Lid %d\r\n",newNode->id);
        h=L;
    }
    if (test==(char)"r"){
    	newNode = (NODE*)nodePtr;
    	nodePtr+=sizeof(NODE);
        (*inNode).r=(NODE*)newNode;//the link between child an parent
        (*newNode).id=((*inNode).id<<1)+2; //the child's node id
        //xil_printf("Rid %d\r\n",newNode->id);
        h=H;
    }
    (*newNode).length=ly; //the length of the child's data array
	y=(long*)tracePtr+FILTER_LENGTH-1;
    tracePtr+=sizeof(long)*(ly+2*(FILTER_LENGTH-1));
	if (newNode->id!=(1<<DEPTH)-1){

	    for (k=0;k<ly;k++){
	        int teller=0, p;
	        y[k]=0;
	        for (p=ih;p<ih+lh;p++){
	            y[k]+=inNode->data[p]*h[teller];
	            teller = teller+1;
	        }
			y[k]/=10000;
	        if (abs(y[k])>(long)tmpMaxAbs){
	                tmpMaxAbs=abs(y[k]);
	                //xil_printf("storre %d\r\n", (short)tmpMaxAbs);
	        }
	        ih=ih+2;
        
	    }
		lag=FILTER_LENGTH-1;
		y-=lag;
		for (k=0;k<lag;k++)
		{
			y[FILTER_LENGTH-2-k]=y[k+lag];
			y[ly+lag+k]=y[ly+FILTER_LENGTH-2-k];
		}
	    (*newNode).maxAbs=tmpMaxAbs;
	    #ifdef DEBUG 
			if ((*newNode).id<20)xil_printf("id %d maxAbs %d length %d\r\n",(*newNode).id,(*newNode).maxAbs,newNode->length);
		#endif
	    //xil_printf("(*inNode).data[22]: %d (*inNode).data[23]: %d\r\n",(*inNode).data[22],(*inNode).data[23]);
	    //xil_printf("y[22]: %d y[23]: %d\r\n",y[22],y[23]);
	    //xil_printf("Wavdec: nodePtr %x %d tracePtr %x %d\r\n", nodePtr,nodePtr,tracePtr,tracePtr);
	    (*newNode).data=y;//the pointer to the child-node's data
		
	    #ifdef DEBUG
			for (k=0;k<5;k++)xil_printf(" %d", (Xint32)y[k+FILTER_LENGTH-1]);
			print("\r\n");
		#endif
	}
	else{
		newNode->maxAbs=0;
		newNode->data=NULL;
		//xil_printf("id %d maxAbs %d\r\n",(*newNode).id,(*newNode).maxAbs);
	}
    if ((*newNode).id<((2<<(DEPTH-1))-1)){
        wavdec((NODE*)newNode,(char)"l");
        wavdec((NODE*)newNode,(char)"r");
    }
    else{
        (*newNode).l=NULL;
        (*newNode).r=NULL;
    }
        
}

/***************************************************************************
* Function for 1D WP analysis with decimation. The function recursively calls
* itself untill the desired depth is reach. A tree-structure which links every
* node together is created as well.
*---------------------------------------------------------------------------
* Input:   inNode   - pointer to a struct representing a node, with its
* 					  belonging data-sequence etc.
*          test    	- this value decides if the LP or HP filter is used
* 		   leafNodes - pointer to pointers of leafNodes
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-02-16
***************************************************************************/
void wavdecRE(NODE *inNode, char test,NODE **leafNodes){
	NODE *newNode;
    short k,ldata=(*inNode).length, lh=FILTER_LENGTH;
	short ly=(short)((lh+ldata-1)>>1); //declaring length of data-array
	short ih=1,lag;
	long *y; //pointer to array of filtered/decimated data
	long *h; //pointer to array of filter-coeffisents
	long tmpMaxAbs=0;
                    
    //checking if the new node is left or right child of parent
    if (test==(char)"l"){
    	(*inNode).l = (NODE*)nodePtr;
    	nodePtr+=sizeof(NODE);
        newNode=(NODE*)(*inNode).l;//the link between child an parent
        (*newNode).id=((*inNode).id<<1)+1; //the child's node id
        //xil_printf("Lid %d\r\n",newNode->id);
        h=L;
    }
    if (test==(char)"r"){
    	newNode = (NODE*)nodePtr;
    	nodePtr+=sizeof(NODE);
        (*inNode).r=(NODE*)newNode;//the link between child an parent
        (*newNode).id=((*inNode).id<<1)+2; //the child's node id
        //xil_printf("Rid %d\r\n",newNode->id);
        h=H;
    }
	newNode->length=ly;
    y=(long*)tracePtr+FILTER_LENGTH-1;
    tracePtr+=sizeof(long)*(ly+2*(FILTER_LENGTH-1));
    //the following loop performs the actual filtering/decimating
    if (newNode->id!=(1<<DEPTH)-1){

	    for (k=0;k<ly;k++){
	        int teller=0, p;
	        y[k]=0;
	        for (p=ih;p<ih+lh;p++){
	            y[k]+=inNode->data[p]*h[teller];
	            teller = teller+1;
	        }
			y[k]/=10000;
	        if (abs(y[k])>(long)tmpMaxAbs){
	                tmpMaxAbs=abs(y[k]);
	                //xil_printf("storre %d\r\n", (short)tmpMaxAbs);
	        }
	        ih=ih+2;
        
	    }
	    lag = FILTER_LENGTH-1;
	    y-=lag;
		for (k=0;k<lag;k++)
		{
			y[FILTER_LENGTH-2-k]=y[k+lag];
			y[ly+lag+k]=y[ly+FILTER_LENGTH-2-k];
		}
	    (*newNode).maxAbs=tmpMaxAbs;
	    //xil_printf("(*inNode).data[22]: %d (*inNode).data[23]: %d\r\n",(*inNode).data[22],(*inNode).data[23]);
	    //xil_printf("y[22]: %d y[23]: %d\r\n",y[22],y[23]);
	    //xil_printf("Wavdec: nodePtr %x %d tracePtr %x %d\r\n", nodePtr,nodePtr,tracePtr,tracePtr);
	    (*newNode).data=y;//the pointer to the child-node's data
	    #ifdef DEBUG 
			if ((*newNode).id<20)xil_printf("id %d maxAbs %d length %d\r\n",(*newNode).id,(*newNode).maxAbs,newNode->length);
	    	for (k=0;k<5;k++){xil_printf(" %d", (Xint32)y[k+49]);}print("\r\n");
		#endif
	}
	else{
		newNode->maxAbs=0;
		newNode->data=NULL;
	}
	if ((*newNode).id<(1<<DEPTH)-1){
		wavdecRE(newNode, (char)"l",leafNodes);
		wavdecRE(newNode,(char)"r",leafNodes);
	}
	else{
		leafNodes[(*newNode).id-(1<<DEPTH)+1]=newNode;
		(*newNode).l=NULL;
		(*newNode).r=NULL;
		newNode->R=0;
		if(newNode->id>(1<<DEPTH)-1){
			newNode->variance=0;
			for (k=0;k<newNode->length;k++){
				newNode->variance+=(newNode->data[k]>>2)*(newNode->data[k]>>2);
			}
		}
	}
}
/***************************************************************************
* Function for 1D WP analysis with decimation for biorthogonal wavelets. 
* The function recursively calls itself untill the desired depth is reach. A
* tree-structure which links every node together is created as well.
*---------------------------------------------------------------------------
* Input:   inNode   - pointer to a struct representing a node, with its
* 					  belonging data-sequence etc.
*          test    	- this value decides if the LP or HP filter is used
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-04-26
***************************************************************************/
void biWavdec(NODE *inNode, char test){
    NODE *newNode;
    short k,ldata=(*inNode).length,lh;
    short ly; //declaring length of data-array
    short ih;
    long *y; //pointer to array of filtered/decimated data
    long *h; //pointer to array of filter-coeffisents
    unsigned long tmpMaxAbs=0;

    //checking if the new node is left or right child of parent
    if (test==(char)"l"){
        (*inNode).l = (NODE*)nodePtr;
    	nodePtr+=sizeof(NODE);
        newNode=(NODE*)(*inNode).l;//the link between child an parent
        (*newNode).id=((*inNode).id<<1)+1; //the child's node id
        //xil_printf("Lid %d\r\n",newNode->id);
        h=L;
        lh=BI_LP_FILTERLENGTH;
        ih=0;
        ly=(short)(ldata+1)/2;
    }
    if (test==(char)"r"){
        newNode = (NODE*)nodePtr;
    	nodePtr+=sizeof(NODE);
        (*inNode).r=(NODE*)newNode;//the link between child an parent
        (*newNode).id=((*inNode).id<<1)+2; //the child's node id
        //xil_printf("Rid %d\r\n",newNode->id);
        h=H;
        lh=BI_HP_FILTERLENGTH;
        ih=2;
        ly=ldata-(short)(ldata+1)/2;
    }
    //newNode->p=inNode;//the link between child an parent
    //the following loop performs the actual filtering/decimating
    if(newNode->id!=(1<<DEPTH)-1)
    {	
    	(*newNode).length=ly; //the length of the child's data array
	    y=(long*)tracePtr+FILTER_LENGTH-1;
	    tracePtr+=sizeof(long)*(ly+2*(FILTER_LENGTH-1));
    
        for (k=0;k<ly;k++){
        	y[k]=0;
            int teller=0, p;
            for (p=ih;p<ih+lh;p++){
                //printf(" \n Og p er: %d", p);
                y[k]=y[k]+inNode->data[p]*h[teller];
                //printf(" \n Og data[p] er: %f og h[lh-in+teller] er: %f ", (*inNode).data[p],h[lh-in+teller]);
                teller = teller+1;
            }
            y[k]/=10000;
            if (abs(y[k])>tmpMaxAbs){
                tmpMaxAbs=abs(y[k]);
            }
            //if(k>65&&k<100)printf(" %ld", (long)y[k]);

            ih=ih+2;
        }
		y-=(FILTER_LENGTH-1);
	    for (k=0;k<4;k++)
	    {
	        y[FILTER_LENGTH-2-k]=y[k+FILTER_LENGTH];
			y[ly+FILTER_LENGTH-1+k]=y[ly+FILTER_LENGTH-3-k];
	    }
	    y+=5;
        (*newNode).maxAbs=tmpMaxAbs;
        (*newNode).data=y;//the pointer to the child-node's data
    }
    else{
        newNode->maxAbs=0;
        newNode->data=NULL;
    }
    if ((*newNode).id<(1<<DEPTH)-1){
        biWavdec(newNode, (char)"l");
        biWavdec(newNode,(char)"r");
    }
    else{
        (*newNode).l=NULL;
        (*newNode).r=NULL;
    }
}

/***************************************************************************
* Function for 1D WP analysis with decimationfor biorthogonal wavelets. The function recursively calls
* itself untill the desired depth is reach. A tree-structure which links every
* node together is created as well.
*---------------------------------------------------------------------------
* Input:   inNode   - pointer to a struct representing a node, with its
* 					  belonging data-sequence etc.
*          test    	- this value decides if the LP or HP filter is used
* 		   leafNodes - pointer to pointers of leafNodes
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-04-26
***************************************************************************/
void biWavdecRE(NODE *inNode, char test,NODE **leafNodes){
				
	NODE *newNode;
    short k,ldata=(*inNode).length,lh;
    short ly; //declaring length of data-array
    short ih;
    long *y; //pointer to array of filtered/decimated data
    long *h; //pointer to array of filter-coeffisents
    unsigned long tmpMaxAbs=0;

    //checking if the new node is left or right child of parent
    if (test==(char)"l"){
        (*inNode).l = (NODE*)nodePtr;
    	nodePtr+=sizeof(NODE);
        newNode=(NODE*)(*inNode).l;//the link between child an parent
        (*newNode).id=((*inNode).id<<1)+1; //the child's node id
        //xil_printf("Lid %d\r\n",newNode->id);
        h=L;
        lh=BI_LP_FILTERLENGTH;
        ih=0;
        ly=(short)(ldata+1)/2;
    }
    if (test==(char)"r"){
        newNode = (NODE*)nodePtr;
    	nodePtr+=sizeof(NODE);
        (*inNode).r=(NODE*)newNode;//the link between child an parent
        (*newNode).id=((*inNode).id<<1)+2; //the child's node id
        //xil_printf("Rid %d\r\n",newNode->id);
        h=H;
        lh=BI_HP_FILTERLENGTH;
        ih=2;
        ly=ldata-(short)(ldata+1)/2;
    } 
    //newNode->p=inNode;//the link between child an parent
    (*newNode).length=ly; //the length of the child's data array
    
    //the following loop performs the actual filtering/decimating
    if(newNode->id!=(1<<DEPTH)-1)
    {
        y=(long*)tracePtr+FILTER_LENGTH-1;
    	tracePtr+=sizeof(long)*(ly+2*(FILTER_LENGTH-1));	
        for (k=0;k<ly;k++){
        	y[k]=0;
            int teller=0, p;
            for (p=ih;p<ih+lh;p++){
                //printf(" \n Og p er: %d", p);
                y[k]=y[k]+inNode->data[p]*h[teller];
                //printf(" \n Og data[p] er: %f og h[lh-in+teller] er: %f ", (*inNode).data[p],h[lh-in+teller]);
                teller = teller+1;
            }
            y[k]/=10000;
            if (abs(y[k])>tmpMaxAbs){
                tmpMaxAbs=abs(y[k]);
            }
            //if(k>65&&k<100)printf(" %ld", (long)y[k]);

            ih=ih+2;
        }
		y-=(FILTER_LENGTH-1);
	    for (k=0;k<4;k++)
	    {
	        y[FILTER_LENGTH-2-k]=y[k+FILTER_LENGTH];
			y[ly+FILTER_LENGTH-1+k]=y[ly+FILTER_LENGTH-3-k];
	    }
	    y+=5;
		(*newNode).maxAbs=tmpMaxAbs;
        (*newNode).data=y;//the pointer to the child-node's data
		#ifdef DEBUG 
			if ((*newNode).id<20)xil_printf("id %d maxAbs %d length %d\r\n",(*newNode).id,(*newNode).maxAbs,newNode->length);
	    	for (k=0;k<5;k++){xil_printf(" %d", (Xint32)y[k+44]);}print("\r\n");
			for (k=0;k<5;k++){xil_printf(" %d", (Xint32)inNode->data[k]);}print("\r\n");
		#endif
        
    }
	else{
		newNode->maxAbs=0;
		newNode->data=NULL;
	}
	if ((*newNode).id<(1<<DEPTH)-1){
		biWavdecRE(newNode, (char)"l",leafNodes);
		biWavdecRE(newNode,(char)"r",leafNodes);
	}
	else{
		leafNodes[(*newNode).id-(1<<DEPTH)+1]=newNode;
		(*newNode).l=NULL;
		(*newNode).r=NULL;
		newNode->R=0;
		if(newNode->id>(1<<DEPTH)-1){
			newNode->variance=0;
			for (k=0;k<newNode->length;k++){
				newNode->variance+=(newNode->data[k]>>2)*(newNode->data[k]>>2);
			}
		}
	}
}
/***************************************************************************
* Function for calculating the mean square distortion and corresponding
* rate for each quantiziser for each node in the tree. The function recursively
* calls itself until all the nodes is reached. The quntizisers is adjusted the 
* dynamic range of the input data.             
*---------------------------------------------------------------------------
* Input:   node   - pointer to a struct representing a node, with its belonging data
*                                         sequence, distortion, rate etc.
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-02-16 
***************************************************************************/
void initRD(NODE *node){
    //xil_printf("initRD: id %d \r\n",node->id);  
    unsigned short i,j,maxRate,N;
	long stepLength;
	long reprLevel, dataj,diff;
    
	i=0;
    //if ((*node).id==0||(*node).id==15) xil_printf("1 id %d initRD:2*maxAbs %ld\r\n",node->id,((*node).maxAbs<<1));
	while ((1<<i)<(node->maxAbs<<1)){
    	i++;//if (i<19)xil_printf("i %d tmp1 %d tmp2 %d\r\n",  i, tmp1, tmp2); 
		
    }
    maxRate=i-1;
    if (maxRate>11)maxRate=11;
    (*node).nmbOfQ=maxRate;
    //if ((*node).id>6&&(*node).id<16) xil_printf("id %d initRD: R_listPtr %x D_listPtr %x maxRate %d 2*maxAbs %d\r\n",  node->id,(Xuint32)R_listPtr,(Xuint32)D_listPtr,maxRate,(short)(2*(*node).maxAbs));
    
    (*node).D_list=(unsigned long*)D_listPtr;
    (*node).R_list=(unsigned char*)R_listPtr;
    D_listPtr+=maxRate*sizeof(unsigned long);//increasing pointers
    R_listPtr+=maxRate*sizeof(char);
    //if ((*node).id>6&&(*node).id<16)xil_printf("3 id %d initRD: R_listPtr %x D_listPtr %x\r\n",  node->id,&R_listPtr,&D_listPtr);
    
	for (i=0;i<maxRate;i++){
    	(*node).R_list[i]=maxRate-i;
		N=1<<(*node).R_list[i];
        
		
		if (node->id==(1<<DEPTH)-1){//exploiting the fact that the lowpass information is not needed
            (*node).R_list[i]=0;
			(*node).D_list[i]=0;
            //xil_printf("er i den hemmelige noden...id: %d %ld\r\n",(*node).id,(Xuint32)node->maxAbs);
        } 
		else if (node->id==0||node->id==1||node->id==3||node->id==7){
			(*node).D_list[i]=1000000000;
		}
        else if (maxRate-i>1){
        	//xil_printf("InitRD if, i %d maxRate %d\r\n", i,maxRate);
			stepLength=(node->maxAbs<<1)/N;
        	#ifdef USE_ESTIMATED_DISTORTION
	            if (stepLength*node->length<=(2000000000/stepLength)*12||stepLength<12){
					(*node).D_list[i]=stepLength*stepLength/12*node->length;
				}
	
				else{
					(*node).D_list[i]=1000000000;
				}
			#else
				#ifdef USE_BIORTHOGONAL_WAVELETS
					(*node).data+=4;
				#else
					(*node).data+=FILTER_LENGTH-1;
				
				#endif
				
	            for (j=0;j<(*node).length;j++){
	                if(node->id==0){
	                    //printf("id: %d nmbOfQ %d\n",(*node).id,(*node).nmbOfQ);
	                }
	                dataj=(long)(*node).data[j];
	                if(node->id==1){
	                    //printf("id: %d dataj %ld\n",(*node).id,dataj);
	                }
	
	                if ((dataj<<1)<(-((*node).maxAbs<<1)+3*stepLength)){
	                    reprLevel=(long)((-(*node).maxAbs)+stepLength);
	                }
	                else if ((*node).data[j]>0){
	                    diff=dataj%(long)stepLength;
	                    if (((diff<<1)/stepLength)<1){
	                        reprLevel=dataj-diff;
	                    }
	                    else{
	                        reprLevel=dataj+(long)stepLength-diff;
	                    }
	                }
	                else{
	                    diff=abs(dataj%(long)stepLength);
	                    if (((diff<<1)/stepLength)<1){
	                        reprLevel=dataj+diff;
	                    }
	                    else{
	                        reprLevel=dataj-(long)stepLength+diff;
	                    }
	                }
	                (*node).D_list[i]+=(unsigned long)(reprLevel-(*node).data[j])*(reprLevel-(*node).data[j]);//accumulating distortion
	                //(*node).D_list[i]+=abs(((float)reprLevel-(*node).data[j]));
	            }
				#ifdef USE_BIORTHOGONAL_WAVELETS
					(*node).data-=4;
				#else
					(*node).data-=(FILTER_LENGTH-1);
				
				#endif
	    	#endif    
			//if (node->id==5)xil_printf("id: %d stepLength %d R_list[i] %d i %d\r\n",(*node).id,(long)stepLength,(*node).R_list[i],i);
			//if(node->id<20)xil_printf("id: %d D_list[i] %ld R_list[i] %d i %d\r\n",(*node).id,(*node).D_list[i],(*node).R_list[i],i);
        }
        else{
        	(*node).R_list[i]=0;
			(*node).D_list[i]=0;
			//xil_printf("InitRD else, id %d length %d %d\r\n",node->id, node->length,node->data[FILTER_LENGTH-1+50]);
			if (node->maxAbs<30000){
				for (j=0;j<node->length;j++){
	            	(*node).D_list[i]+=node->data[j+FILTER_LENGTH-1]*node->data[j+FILTER_LENGTH-1];//accumulating distortion
	                //if (node->id<5&&j<36&&j>30)xil_printf("N %d data %d acc %d\r\n", N,node->data[j+FILTER_LENGTH-1],(*node).D_list[i]);
	            }
				//(*node).D_list[i]<<=2;	
			}
			else{
				(*node).D_list[i]=1000000000;
			}
			//xil_printf("id: %d D_list[i] %ld R_list[i] %d i %d\r\n",(*node).id,(*node).D_list[i],(*node).R_list[i],i);
            //if (node->id<5)xil_printf("id: %d D_list[i] %d R_list[i] %d i %d\r\n",(*node).id,acc,(*node).R_list[i],i);
        }
    }                                   
    if ((*node).l!=NULL){
    	//xil_printf("venstre\r\n");
    	initRD((NODE*)(*node).l);
    }

    if ((*node).r!=NULL){
    	//xil_printf("hoeyre\r\n");
    	initRD((NODE*)(*node).r);
    }
}

/***************************************************************************
* Function for calculating the cost for a given lambda. The function also 
* calculates the total rate for each node in bits. The function recursively calls
* itself so that all nodes is reached.             
*---------------------------------------------------------------------------
* Input:   node   - pointer to a struct representing a node, with its belonging data
*                                         sequence, distortion, rate etc.
*                   lambda  - lambda                         
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-02-19 
***************************************************************************/
void setCost(NODE *node, long *lambda){
	//long tic = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1);
	long tic;
    (*node).J=2000000000;
    
    short i;
    long tmpJ;
    //if (node->id<15)xil_printf("ID = %d nmbOfQ= %d \r\n",node->id, (short)(*node).nmbOfQ);
	if(node->id==0||node->id==1||node->id==3){
		(*node).R=1;
	}
	else{
	    for (i=0;i<(*node).nmbOfQ;i++){
			if ((*node).R_list[i]>0){
				tmpJ=(*node).D_list[i]+(*lambda)*((*node).R_list[i]*(*node).length+OVERHEAD);
			}
	        else{
	            tmpJ=(*node).D_list[i];
	        }

			#ifdef DEBUG 
				if (node->id<2)xil_printf("ID = %d R_List= %d D_List %d tmpJ %d\r\n",node->id, (*node).R_list[i],(long)(*node).D_list[i],(long)tmpJ);
			#endif
	        if (tmpJ<(*node).J){
	            //if (node->id<15&&i<2)xil_printf("ID = %d nmbOfQ= %d R %d length %d %d\r\n",node->id, (short)(*node).nmbOfQ,node->R,node->length,node->totR);	
	            (*node).J=tmpJ;
	            (*node).R=(*node).R_list[i];
	            (*node).D=(*node).D_list[i];
	            if((*node).R_list[i]>0){
	                (*node).totR=((short)(*node).R)*(*node).length+OVERHEAD;
				
	            }
	            else{
	                (*node).totR=0;
				}
	        }
	    }
	}
    //if (node->id<15)xil_printf("ID = %d R %d length %d totR %d\r\n",node->id,node->R,node->length,node->totR);
    //xil_printf("lambda = %d ID = %d Cost %ld TotR %d R %d\r\n" ,(long)*lambda, (*node).id,(long)(*node).J,(long)(*node).totR,node->R);
	
    if (((*node).l!=NULL)&&((*node).r!=NULL)){
        setCost((NODE*)(*node).l, lambda);
        setCost((NODE*)(*node).r, lambda);
    }
	//tic = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1);
	//tic = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1)-tic;
	//xil_printf("Id %d tic %d\r\n",node->id,tic);
	
	
}



/***************************************************************************
* Function for finding the best subtree in a rate-distortion sense. Instead of
* brute-forcing all possible combinations of subtrees, dynamic programming is
* employed.               
*---------------------------------------------------------------------------
* Input:   node   - pointer to a struct representing a node, with its belonging data
* 					sequence, distortion, rate etc.
* 			nmb	  - total number of nodes in the tree                          
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-02-21 
***************************************************************************/
void findSubTree(NODE* node, short nmb){
	/*long tic;
	if(node->id==0){
		tic = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1);
	}*/
	NODE *lchild, *rchild;
	lchild = (NODE*)(*node).l;
	rchild = (NODE*)(*node).r;
	if ((*node).id<nmb-(1<<DEPTH)-(1<<(DEPTH-1))){
        findSubTree((NODE*)lchild,nmb);
        findSubTree((NODE*)rchild,nmb);
    }
    else{
        lchild->split=FALSE;
        rchild->split=FALSE;
    }

    //printf("costL= %f costR= %f costP= %f\n",(*lchild).J,(*rchild).J,(*node).J);
    if ((((*lchild).J+(*rchild).J)<(*node).J)&&((*lchild).R>0||(*rchild).R>0)){

        (*node).split=TRUE;
        (*node).totR=(*rchild).totR+(*lchild).totR;
        (*node).J=(*rchild).J+(*lchild).J;
        (*node).D=(*rchild).D+(*lchild).D;
    }
    else{
        (*node).split=FALSE;
        //printf("No split: PID= %d LID: %d RID: %d\n",(*node).id,(*lchild).id,(*rchild).id);
    }
	//xil_printf("ID %d split %d totR %d D %d R %d length %d lR %d rR %d\r\n",(*node).id,(*node).split,(*node).totR,(long)(*node).D,(long)node->R,node->length,(long)lchild->R,(long)rchild->R);
	/*if (node->id==0){
		tic = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1)-tic;
		xil_printf("Id %d tic %d\r\n",node->id,tic);
	}*/
}

/***************************************************************************
* Function for finding the best subtree in a rate-distortion sense. Instead of
* brute-forcing all possible combinations of subtrees, dynamic programming is
* employed.
*---------------------------------------------------------------------------
* Input:   root   - pointer to the root-node, with its belonging data
* 					sequence, distortion, rate etc.
* 			nmb	  - total number of nodes in the tree
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-05-02
***************************************************************************/
void findBestSubTree(NODE* root, short nmb){
	long RUL,RLL,DUL,DLL,lambda,lambdaUL=100000, lambdaLL=0;
    short testLambda=FALSE,i=0;
    
	while((testLambda==FALSE)&&(i<15)){
		//finding the initial values of the rate/distortion for the upper/lower limit of lambda
		setCost((NODE*)root,&lambdaUL);
		findSubTree((NODE*)root, nmb);
		RUL=(*root).totR;
		DUL=(*root).D;
		#ifdef DEBUG
			xil_printf("RUL %d DUL %d totR %d\r\n",(short)RUL,(long)DUL,root->totR);
		#endif
		setCost((NODE*)root,&lambdaLL);
		findSubTree((NODE*)root, nmb);
		RLL=(*root).totR;
		DLL=(*root).D;
		#ifdef DEBUG
			xil_printf("RLL %d DLL %d totR %d\r\n",(short)RLL,(long)DLL,root->totR);
		#endif
		lambda=(DUL-DLL)/(RLL-RUL)+1;
		//xil_printf("lambda %d\r\n",(short)lambda);
		//finding the best cost/quantiziser/rate for this lambda for every node
		setCost((NODE*)root,&lambda);
		// finding the best basis in a R-D sense
		findSubTree((NODE*)root, nmb);
		if ((unsigned long)RUL==(*root).totR){
			testLambda=TRUE;	
		}
		else if ((*root).totR>RBUDGET){
			lambdaLL= lambda;		
		}
		else {
			lambdaUL=lambda;
		}
		i++;
		
		//float n1=1000.0*lambdaUL,n2=1000.0*lambdaLL,n3=1000.0*lambda;
		#ifdef DEBUG
			xil_printf("lambdaUL: %d lambdaLL: %d lambda %d i: %d totR: %d \r\n",(unsigned long)lambdaUL,(unsigned long)lambdaLL,(unsigned long)lambda,i,(*root).totR); 
		#endif
	}
}

/***************************************************************************
* Function for creating the structs representing the leaf-nodes whos data is 
* going to be quantizised. The leaf structs in addition represents all the 
* relevant metadata for decoding.              
*---------------------------------------------------------------------------
* Input:   node   - pointer to a struct representing a node, with its belonging data
*                                         sequence, distortion, rate etc.
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-02-23 
***************************************************************************/
void createLeaves(NODE* node){
        if ((*node).split==TRUE){
                createLeaves((NODE*)(*node).l);
                createLeaves((NODE*)(*node).r);
        }
        else{
                LEAF * leaf;
                leaf=(LEAF*)leafPtr;
                leafPtr+=sizeof(LEAF);
                (*leaf).id=(unsigned char)(*node).id;
                (*leaf).R=(unsigned char)(*node).R;
                (*leaf).length=(unsigned short)(*node).length;
                (*leaf).maxAbs=(unsigned short)((*node).maxAbs);
				//xil_printf("Id %d R %d split %d\r\n", leaf->id,leaf->R,node->split);
                if ((*leaf).R>0){
					#ifdef DEBUG 
						xil_printf("Id %d R %d\r\n", leaf->id,leaf->R);
					#endif
                    unifQuantDead((NODE*)node, (LEAF*)leaf);
		
                    rle(leaf);
                    writeLeaf((LEAF*)leaf);
                        
                }
				#ifdef DEBUG 
					xil_printf("\n ID %d maxAbs %d\r\n",leaf->id,leaf->maxAbs);
				#endif
        }
}

/***************************************************************************
* Function for uniform quantization and. The indicies of the quantizised data are
* put in the leaf's data-array.               
*---------------------------------------------------------------------------
* Input:   node   - pointer to a struct representing a node, with its belonging data
*                                         sequence, distortion, rate etc.
*                    leaf   -        pointer to a struct representing a leaf in the optimal subtree
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-02-24 
***************************************************************************/
void unifQuant(NODE* node, LEAF * leaf){
	//tic = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1);
	unsigned short N,i,lag;
	long tmp,tmp2,stepLength;
    N=2<<((short)node->R-1);
    stepLength = (node->maxAbs<<1)/N;
    leaf->data=(unsigned short*)qdataPtr;
    qdataPtr+=leaf->length*sizeof(unsigned short);
	#ifdef DEBUG 
		xil_printf("\nID %d R %d maxA %d length %d\r\n",leaf->id,leaf->R,leaf->maxAbs,leaf->length);
	#endif
    #ifdef USE_BIORTHOGONAL_WAVELETS
    	lag=4;
    #else
    	lag=FILTER_LENGTH-1;
    #endif
    node->data+=lag;
    for (i=0;i<(*leaf).length;i++){
        if(node->data[i]>0){
            tmp=node->data[i]+(stepLength>>1);
            leaf->data[i]=(unsigned short)(tmp/stepLength)+(N>>1)-1;
			if (leaf->data[i]>=(1<<leaf->R)){
				leaf->data[i]=(1<<leaf->R)-1;
			}
        }
        else{
        	
            tmp=node->data[i]-(stepLength>>1);
            tmp2=tmp/stepLength+(N>>1)-1;
            
            if(tmp2>0){
                leaf->data[i]=(unsigned short)tmp2;
            }
            else{
                leaf->data[i]=0;
            }
       }
	   #ifdef DEBUG 
	   		xil_printf(" %d",leaf->data[i]);
	   		if(node->id==17){xil_printf(" %ld",node->data[i]);}
		#endif
     }
	 #ifdef DEBUG 
	 	xil_printf("\r\n");
	 #endif
     //if (DEBUG)xil_printf("\r\nQuantized values for id %d length %d rate %d\r\n",leaf->id,leaf->length,leaf->R);
     /*for (i=0;i<leaf->length;i++){
     	if (DEBUG)xil_printf(" %d", leaf->data[i]);
     }*/
	 //tic = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1)-tic;
		//xil_printf("Id %d tic %d\r\n",leaf->id,tic);
}

/***************************************************************************
* Function for uniform quantization with deadzone. The indicies of the quantizised data are
* put in the leaf's data-array.
*---------------------------------------------------------------------------
* Input:   node   - pointer to a struct representing a node, with its belonging data
* 					sequence, distortion, rate etc.
* 		   leaf   -	pointer to a struct representing a leaf in the optimal subtree
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-04-30
***************************************************************************/
void unifQuantDead(NODE* node, LEAF * leaf){
    long stepLengthOver,stepLengthUnder,stepLength,tmp,tmp2,tmp3;
    short N,i,zeroRepresentation,lag;
    N=(short)1<<node->R;
    #ifdef USE_BIORTHOGONAL_WAVELETS
    	lag=4;
    #else
    	lag=FILTER_LENGTH-1;
    #endif
    node->data+=lag;
    zeroRepresentation=(N>>1)-1;
    stepLength = 2*node->maxAbs/N;
    tmp3=((stepLength*DEADZONE_NUM)>>1)/DEADZONE_DEN;//be shure that this calculation only is performed once...
    stepLengthOver=(node->maxAbs-tmp3)/(N>>1);
    stepLengthUnder=(node->maxAbs-tmp3)/((N>>1)-1);
    
    leaf->data=(unsigned short*)qdataPtr;
    qdataPtr+=leaf->length*sizeof(unsigned short);
    for (i=0;i<(*leaf).length;i++){
    	if(abs(node->data[i])<tmp3){
    		leaf->data[i]=zeroRepresentation;
    	}
        else if(node->data[i]>0){
            tmp=node->data[i]-tmp3;
            leaf->data[i]=(unsigned short)zeroRepresentation+(tmp/stepLengthOver)+1;
			if (leaf->data[i]>=N){
				leaf->data[i]=N-1;
			}
            
            //if (leaf->id==25) printf("shift %d leaf->R %d\n",(2<<(short)(leaf->R-2))-1,leaf->R);
        }
        else{
        	
            tmp=node->data[i]+tmp3;
            tmp2=(short)zeroRepresentation+(tmp/stepLengthUnder)-1;
            //printf(" %d %f %f ",(short)(tmp/stepLength),tmp, tmp2);
            if(tmp2>0){
                leaf->data[i]=(unsigned short)tmp2;
            }
            else{
                leaf->data[i]=0;
            }
            

        }
      	#ifdef DEBUG 
			xil_printf(" %d",leaf->data[i]);
	   		if(node->id==17){xil_printf(" %ld",node->data[i]);}
		#endif
		
     }
	 #ifdef DEBUG 
	 	xil_printf("\r\n");
	 #endif
     //printf("\n ID %d R %d maxA %d length %d nmbZeros %d\n",leaf->id,leaf->R,leaf->maxAbs,leaf->length,leaf->nmbZeros);
}

/***************************************************************************
* Function for run length encoding               
*---------------------------------------------------------------------------
* Input:   leaf   -        pointer to a struct representing a leaf in the optimal subtree
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-03-19 
***************************************************************************/
void rle(LEAF* leaf){
		//tic = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1);
		short rl=-1;
        unsigned short *y,past,yi=1,i,tmpR;
        y=(unsigned short*)rleDataPtr;
        y[0]=leaf->data[0];
        past = leaf->data[0];
        tmpR=leaf->R;
        if (leaf->R<4){
                tmpR=4;
        }
		

        for (i=1;i<leaf->length;i++){
                if (past==leaf->data[i]){
                        rl++;
                        if (rl>=(2<<(tmpR-1))-1){
                                y[yi]=past;
                                y[yi+1]=rl;
                                yi+=2;
                                rl=-1;
                                past=1<<14;
                        }
                }
                else if ((past!=leaf->data[i])&&rl>-1){
                        y[yi]=past;
                        y[yi+1]=rl;
                        y[yi+2]=leaf->data[i];
                        yi+=3;
                        rl=-1;
                        past=leaf->data[i];        
                }
                else{
                        y[yi]=leaf->data[i];
                        yi++;
                        past=leaf->data[i];
                }
        
        }
		
        if (rl>-1){
                y[yi]=past;
                y[yi+1]=rl;
                yi+=2;
        }
        leaf->lengthRl=yi;
        rleDataPtr+=sizeof(unsigned short)*yi;
        leaf->rlData=(unsigned short*)y;
        if ((leaf->length*leaf->R)>(leaf->lengthRl*tmpR+10)){
                leaf->isRLenc=TRUE;
				#ifdef DEBUG 
					xil_printf("Id %d rle\r\n ",leaf->id);
				#endif
                //leaf->R=tmpR;
        }
        else{
                leaf->isRLenc=FALSE;
				#ifdef DEBUG
					xil_printf("Id %d no rle\r\n ",leaf->id);
				#endif
        }
		
		
		#ifdef DEBUG
			xil_printf("\nAfter rle: id %d lengthRl %d length %d\r\n ",leaf->id,leaf->lengthRl,leaf->length);
			for (i=0;i<leaf->lengthRl;i++){
				xil_printf(" %d ",leaf->rlData[i]);
				
			}
		#endif
		//tic = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1)-tic;
		//xil_printf("Id %d tic %d\r\n",leaf->id,tic);
        /*xil_printf("\r\nRLE encoded values for id %d lengthRl %d\r\n",leaf->id,leaf->lengthRl);
	     for (i=0;i<leaf->lengthRl;i++){
	     	xil_printf(" %d", leaf->rlData[i]);
	     }*/
}

/***************************************************************************
* Function for writing a leaf to an outputfile               
*---------------------------------------------------------------------------
* Input:   leaf   -        pointer to a struct representing a leaf in the optimal subtree
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-02-24 
***************************************************************************/
void writeLeaf(LEAF* leaf){
        unsigned char bitbuffer,j,y1,y2,tmpR;
        unsigned short mask,x,i,*y,length;
		long tic;
		
		//tic = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1);
        /********************************************************************************
         * Writing header (a total of 26 bits for each leaf):
         * -maxAbs is the absolute value of the biggest value in the dataset, 16 bits
         * -id is the unique id of the node so that it is possible to reconstruct the signal
         *         at the decoder, represented by 6 bits 
         * -Rate is the number og bits each sample is represented with, this field is 
         *         represented with 4 bits
         * ********************************************************************************/
        //write maxAbs (16 bits)
        y1=0;y2=0;
        x=leaf->maxAbs;
        y2=x;
        x>>=8;
        y1=x;

        //XGpio_mSetDataReg(XPAR_RS232_CONTROL_BASEADDR,1,0);
        //XUartLite_SendByte((unsigned long) XPAR_RS232_BASEADDR,y1);
        //XUartLite_SendByte((unsigned long) XPAR_RS232_BASEADDR,y2);
        //write id (6 bits)
        mask=1<<5;
        bitbuffer=0;
        x=(unsigned short)leaf->id;
        y1=0;
        for (i=0;i<6;i++){
                y1=(((x&mask)>>5)|y1);
                bitbuffer++;
                y1<<=1;
                x<<=1;
        }
        //write rate(4 bits)
        mask=1<<3;
        x=leaf->R;
        for (i=0;i<4;i++){
                y1=(((x&mask)>>3)|y1);
                bitbuffer++;
                if (bitbuffer==8){
                        //XUartLite_SendByte((unsigned long) XPAR_RS232_BASEADDR,y1);
                        y1=0;
                        bitbuffer=0;
                }
                y1<<=1;
                x<<=1;
        }

        
        //write isRLenc
        mask=1;
        x=leaf->isRLenc;
        y1=x|y1;
        bitbuffer++;
        y1<<=1;
        x<<=1;
        //write length of rle data if used
        if(leaf->isRLenc==TRUE){
                //if(leaf->id==16)printf("2: R %d y1 %d x %d bitbuffer %d\n",leaf->R,y1,x,bitbuffer);
        
                mask=1<<9;
                x=leaf->lengthRl;
                for (i=0;i<10;i++){
                        y1=(((x&mask)>>9)|y1);
                        bitbuffer++;
                        if (bitbuffer==8){
                                //XUartLite_SendByte((unsigned long) XPAR_RS232_BASEADDR,y1);
                                y1=0;
                                bitbuffer=0;
                        }
                        y1<<=1;
                        x<<=1;                        
                }
        }
        
        /***********************************************************
         * Writing quantizised data to output file
         * *********************************************************/
         if(leaf->isRLenc==TRUE){
                 y=leaf->rlData;
                 length=leaf->lengthRl;
         }
         else{
                 y=leaf->data;
                 length=leaf->length;
         }
         if(leaf->R<4)
		{
			tmpR=4;
		}
		else
		{
			tmpR=leaf->R;
		}
		mask=1<<(tmpR-1);                
        for (i=0;i<length;i++)
        {
                x=(unsigned short)y[i];
                //if(leaf->id==8 && i<10)printf("\nx %d\n",x);
                for (j=0;j<tmpR;j++){
                        //if(leaf->id==8 && i<10)printf("1((x&mask)>>(leaf->R-1)) %d\n",((x&mask)>>(leaf->R-1)));
                        y1=(((x&mask)>>(tmpR-1))|y1);
                        //if(leaf->id==8 && i<10)xil_printf("2: R %d y1 %x x %d bitbuffer %d y %d\n",leaf->R,y1,x,bitbuffer,y[i]);
                        bitbuffer++;
                        if (bitbuffer==8){
                                //XUartLite_SendByte((unsigned long) XPAR_RS232_BASEADDR,y1);
                                y1=0;
                                bitbuffer=0;
                        }
                        y1<<=1;
                        x<<=1;
                }
        }
        
        //writing out the bits stored in y1
        if (bitbuffer>0){
                y1<<=(8-bitbuffer-1);
                //XUartLite_SendByte((unsigned long) XPAR_RS232_BASEADDR,y1);;
        } 
		//tic = XTmrCtr_mGetTimerCounterReg(XPAR_OPB_TIMER_0_BASEADDR,1)-tic;
		//xil_printf("Id %d tic %d\r\n",leaf->id,tic); 
}

/***************************************************************************
* Function for writing length of original signal after cut              
*---------------------------------------------------------------------------
* Input:   length   -    length of signal
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-04-21 
***************************************************************************/
void writeLength(unsigned short length){

	unsigned char y1,y2;
	y1=0;y2=0;
    y2=length;
    length>>=8;
    y1=length;
	//XUartLite_SendByte((unsigned long) XPAR_RS232_BASEADDR,y1);
	//XUartLite_SendByte((unsigned long) XPAR_RS232_BASEADDR,y2);
}

/***************************************************************************
* Function for creating the structs representing the leaf-nodes whos data is
* going to be quantizised. The leaf structs in addition represents all the
* relevant metadata for decoding.
*---------------------------------------------------------------------------
* Input:   nodes   - pointer to pointers of structs representing a node, with its belonging data
* 					sequence, distortion, rate etc.
* 		   fOut	   - pointer to output stream	
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-04-29
***************************************************************************/
void writeLeaves(NODE** nodes){
    short i;
    NODE* node;
    xil_printf("In writeLeaves\n");
    for (i=1;i<(1<<DEPTH);i++){
    	node=(NODE*)nodes[i];
	    LEAF * leaf;
	    leaf=(LEAF*)leafPtr;
        leafPtr+=sizeof(LEAF); 
	    (*leaf).id=(unsigned char)(*node).id;
	    (*leaf).R=(unsigned char)(*node).R;
	    if(leaf->R==1){
	    	leaf->R=2;//want minimum four bits
	    	node->R=2;
	    }
	    (*leaf).length=(unsigned short)(*node).length;
	    (*leaf).maxAbs=(unsigned short)(*node).maxAbs;
	    if ((*leaf).R>0){
	    	unifQuantDead((NODE*)node, (LEAF*)leaf);
	    	
	        
	        rle(leaf);
	
	        writeLeaf((LEAF*)leaf);
	
	    }
	    xil_printf("ID %d maxAbs %d\r\n",leaf->id,leaf->maxAbs);
    }
}
/***************************************************************************
* Function for allocating bits. Greeda algorithm, gives bits to subband with highest variance
* and then dividing its variance with a certain factor
*---------------------------------------------------------------------------
* Input:   nodes   -	pointer to pointers of structs representing a node, with its belonging data
* 					sequence, distortion, rate etc.
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-04-29
***************************************************************************/
void allocateBits(NODE** nodes){
	short test = TRUE,i,rate,index,numerator;
	long tmpMax;
	rate=0;
	numerator=0;
	while (test){
		//finding the highest variance
		tmpMax=0;
		index=-1;
		 
		for (i=1;i<(1<<DEPTH);i++){
			if (nodes[i]->variance>tmpMax){
				index=i;
				tmpMax=nodes[i]->variance;
			}
		}
		nodes[index]->R++;
		nodes[index]->variance=(unsigned long)nodes[index]->variance/(NUM_VAR/DEN_VAR);
		if ((1<<(nodes[index]->R+2)>nodes[index]->maxAbs)||(nodes[index]->R==15))
		{
			nodes[index]->variance=0;
			numerator++;
				if(numerator==(1<<DEPTH)-1){
					test=FALSE;	
				}
		
		}
		rate+=nodes[index]->length;
		if(rate>=RBUDGET){
			test=FALSE;
		}
	}
}
