/***************************************************************************
*                             INCLUDED FILES
***************************************************************************/
#include <stdio.h>
#include <math.h>
#include <malloc.h>

/***************************************************************************
*                                CONSTANTS
***************************************************************************/
#define DEBUG 1
#define TRUE 1
#define FALSE 0 
#define OVERHEAD 26
#define FILTER_LENGTH 10
#define BI_LP_FILTERLENGTH 9
#define BI_HP_FILTERLENGTH 7
#define INITIAL_LENGTH_OF_TRACES 2048
/***************************************************************************
*       DEFINES which decides the properties of the encoder. 
***************************************************************************/
#define USE_ESTIMATED_DISTORTION //estimated or real...Only used if BEST_WP_BASES
//#define BEST_WP_BASES //use this if BEST_WP_BASES algorithm should be used, otherwise a simple bit-allocation scheme is used
#define NUM_VAR 4 //numerator and denominator for the factor which the variance is divided on
#define DEN_VAR 1
#define RBUDGET 2000
#define DEPTH 4 //the decomposition level
#define HIGH_RATE //use this if high-rate traces are teste, otherwise the low-corrosion-rate traces are employed



/*The values below must be the same in encoder and decoder!!!*/
#define DEADZONE_NUM 2//numerator and denominator for the factor which the deadzone is multiplied with
#define DEADZONE_DEN 1

//#define USE_BIORTHOGONAL_WAVELETS


/***************************************************************************
*                                MACROS
***************************************************************************/
#define max(a,b) ((a)<(b)?(b):(a))
#define min(a,b) ((a)<(b)?(a):(b))
#define abs(a) ((a)<(0)?(-a):(a))


/***************************************************************************
*                            TYPE DEFINITIONS
***************************************************************************/
typedef struct {//defining the nodes in tree
    struct NODE *l,*r;//l,r is the left/right child respectively
    short id, length,split,nmbOfQ;
    long *data, maxAbs;
    unsigned char *R_list;
    unsigned long *D_list,totR,J,R,D,variance;
} NODE;
typedef struct {//defining the final leaves
    unsigned short maxAbs,length,*data,lengthRl,*rlData;
    unsigned char id,R,isRLenc;
} LEAF;


/***************************************************************************
*                               PROTOTYPES
***************************************************************************/
void wavdec(NODE *inNode,  char c);
void biWavdec(NODE *inNode, char test);
void initRD(NODE *node);
void setCost(NODE *node, long *lambda);
void findSubTree(NODE* node, short nmb);
void createLeaves(NODE* node, FILE *fOut);
void unifQuantDead(NODE* node, LEAF * leaf);
void writeLeaf(LEAF* leaf,FILE* fOut);
void rle(LEAF* leaf);
void writeUS(unsigned short us, FILE* fOut);
void writeLeaves(NODE** nodes, FILE *fOut);
void allocateBits(NODE** nodes);
void wavdecRE(NODE *inNode,  char c,NODE** nodes);
void biWavdecRE(NODE *inNode, char test,NODE** nodes);
void findBestSubTree(NODE* root, short nmb);
void getTree(NODE* node);
void setTree(NODE* node);


/***************************************************************************
*                            GLOBAL VARIABLES
***************************************************************************/
//const float HD= [-0.0645388826286971 0.0406894176091641 0.418092273221617 -0.788485616405583 0.418092273221617 0.0406894176091641 -0.0645388826286971];
//const float LD= [0.037828455507264 -0.0238494650195568 -0.110624404418437 0.377402855612831 0.852698679008894 0.377402855612831 -0.110624404418437 -0.0238494650195568 0.037828455507264];
#ifdef USE_BIORTHOGONAL_WAVELETS
	long H[7]= {-645, 406, 4180, -7884, 4180, 406, -645};
	long L[9]= {378, -238, -1106, 3774, 8526, 3774, -1106, -238, 378};

#else
	long H[10]={33, 125, -62,-775, -322, 2422, 1384,-7243, 6038, -1601};
	long L[10]={1601, 6038, 7243, 1384, -2422, -322, 775, -62, -125, 33};
#endif
int kr=0,loev[31],rater[31];
/***************************************************************************
*                                FUNCTIONS
***************************************************************************/
int main(int argc,char *argv[]) {  /* BEGIN MAIN */
    short i ,t;
    short nbNodes=0,*inp,index;
    long *input;//storage-array for input-data
    unsigned short lengthOfSignal;
    long tmpMax;
    
    //char string[]="200512151138.dat";//filename
    #ifdef HIGH_RATE
		char string[]="hrate_orig.dat";
	#else
		char string[]="500_original.dat";
	#endif
    
    
    NODE **leafNodes=calloc((1<<DEPTH),sizeof(NODE*));
    
    FILE *fInput, *fOut;
    for (i=0;i<=DEPTH;i++){nbNodes+=pow(2,i);}
    NODE *root;
    //open file for reading
    if ((fInput = fopen(string,"rb")) == NULL){
        fprintf(stderr,"Cannot open file for reading\n");
        exit(1);
    }
	//open file for writing 
    if ((fOut = fopen("testOut.dat", "wb")) == NULL)
    {
        fprintf(stderr,"Cannot open file for writing\n");
        exit(1);
    }
	input=(long*)calloc(2048,sizeof(long));
	inp=(short*)calloc(2048,sizeof(short));
	for (kr=0;kr<500;kr++){
	    //printf(" %d ",kr);
	    //read input
		t=fread(inp,sizeof(short),2048,fInput);
    		
		//finding the trace's max which is assumed to be the first peak
		tmpMax=-33000;
	    for (i=0;i<INITIAL_LENGTH_OF_TRACES;i++){
	        input[i]=(long)inp[i];
	        if (input[i]>tmpMax){
				tmpMax=input[i];
	            index=i;
	        }
	    }
	    //want even number
	    //printf(" %d", index);
		if (index%2==1){
			index++;
		}
		
	    //the trace must start before the first peak
		index-=300;
		if (index<0)
		{
			index=0;
		}
		//index=740;
		
		root=(NODE*)calloc(1,sizeof(NODE));
		(*root).length=INITIAL_LENGTH_OF_TRACES-index;
		root->data=(long*)calloc(root->length,sizeof(long));
		//printf(" %d", root->length);
		//printf("length before preprocessing %d index %d\n", (*root).length,index);
		root->maxAbs=30000;
        for (i=0;i<(*root).length;i++){
			root->data[i]=input[i+index];
	    }
	    //for (i=0;i<10;i++){printf(" %ld", root->data[i]);}
	    (*root).id=0;
	    if(DEPTH <5){
	        
		    root->length=(root->length+1)>>1;
		    //printf(" %d", root->length);
	        if(root->length%2==1){
	        	root->length--;
	        	root->data++;
	        }
	        for (i=0;i<root->length;i++){
	            root->data[i]=(root->data[i<<1]>>1)+(root->data[(i<<1)+1]>>1);
	            //xil_printf(" %d", root->data[i+FILTER_LENGTH-1]);
	        }
	            
	        
	    }
	    lengthOfSignal=root->length;
	    if (kr==93||kr==94||kr==95){
	    	printf("length after preprocessing %d\n", (*root).length);
	    }
	
	    //for (i=0;i<100;i++){printf(" %ld", (*root).data[i]);}
	
	    /*running wavedec-function for left and right child, i.e. filtering with
	    * high-pass and low-pass and decimating with a factor two. The tree-structure is created*/
	    #ifdef BEST_WP_BASES
		    /*running wavedec-function for left and right child, i.e. filtering with
		    * high-pass and low-pass and decimating with a factor two. The tree-structure is created*/
		    #ifdef USE_BIORTHOGONAL_WAVELETS
		    	biWavdec(root,(char)"l");
		    	biWavdec(root,(char)"r");	
		    #else
		    	wavdec(root,(char)"l");
		    	wavdec(root,(char)"r");
		    #endif
		    //creating the R-D points for all the nodes in the tree
		    initRD(root);
		    //printf("k %d ",k);
	        findBestSubTree(root,nbNodes);
	    #else
	    	/*running wavedec-function for left and right child, i.e. filtering with
		    * high-pass and low-pass and decimating with a factor two. The tree-structure is created*/
		    #ifdef USE_BIORTHOGONAL_WAVELETS
		    	biWavdecRE(root,(char)"l",leafNodes);
		    	biWavdecRE(root,(char)"r",leafNodes);	
		    #else
		    	wavdecRE(root,(char)"l",leafNodes);
		    	wavdecRE(root,(char)"r",leafNodes);
		    #endif
		    //printf("k %d ",k);
		    //creating the R-D points for all the nodes in the tree
		    allocateBits(leafNodes);
		#endif
		    
	    
	    //write header after cut
	    writeUS(lengthOfSignal, fOut);
	    
	    #ifdef BEST_WP_BASES
		    //Creating the leaf-nodes which shall be written to the outputfile
		    /*if (kr==0){
		    	for(i=0;i<31;i++){
		    		loev[i]=0;
		    		rater[i]=0;
		    	}
		    	getTree(root);
		    }
		    else{
		    	setTree(root);
		    }*/
		    createLeaves(root, fOut);
		#else
			writeLeaves(leafNodes, fOut);
			
		#endif
	    //write zero so the decoder knows that the trace is finished encoded
	    writeUS(0, fOut);
	}
    fclose(fInput);
    fclose(fOut);

    return 0;
}



/***************************************************************************
* Function for 1D WP analysis with decimation for biorthogonal wavelets. 
* The function recursively calls itself untill the desired depth is reach. A
* tree-structure which links every node together is created as well.
*---------------------------------------------------------------------------
* Input:   inNode   - pointer to a struct representing a node, with its
* 					  belonging data-sequence etc.
*          test    	- this value decides if the LP or HP filter is used
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-04-26
***************************************************************************/
void biWavdec(NODE *inNode, char test){
    NODE *newNode=calloc(1,sizeof(NODE));
    short k,ldata=(*inNode).length,lh;
    short ly; //declaring length of data-array
    short ih;
    long *y; //pointer to array of filtered/decimated data
    long *h; //pointer to array of filter-coeffisents
    long *symData;
    unsigned long tmpMaxAbs=0;

    //checking if the new node is left or right child of parent
    if (test==(char)"l"){
        (*inNode).l=calloc(1,sizeof(NODE));
        newNode=(NODE*)(*inNode).l;//the link between child an parent
        (*newNode).id=2*(*inNode).id+1; //the child's node id
        //printf(" \n Det ble lavpass");
        h=L;
        lh=BI_LP_FILTERLENGTH;
        ih=0;
        ly=(short)(ldata+1)/2;
    }
    if (test==(char)"r"){
        (*inNode).r=calloc(1,sizeof(NODE));
        newNode=(NODE*)(*inNode).r;//the link between child an parent
        (*newNode).id=2*(*inNode).id+2; //the child's node id
        //printf(" \n Det ble hypass");
        h=H;
        lh=BI_HP_FILTERLENGTH;
        ih=2;
        ly=ldata-(short)(ldata+1)/2;
    }
    //newNode->p=inNode;//the link between child an parent
    (*newNode).length=ly; //the length of the child's data array
    //for (k=0;k<ly;k++){printf(" %f", data[k]);}
    y=(long*)calloc(ly,sizeof(long));
    //printf(" \n Lengde av y: %d", ly);

    //the following loop performs the actual filtering/decimating
    if(newNode->id!=(1<<DEPTH)-1)
    {
        symData=(long*)calloc(ldata+8,sizeof(long));
	    for (k=0;k<4;k++)
	    {
	        symData[3-k]=inNode->data[k+1];
	        symData[ldata+4+k]=inNode->data[ldata-2-k];
	    }
	    for (k=0;k<ldata;k++)
	    {
	        symData[k+4]=inNode->data[k];
	    }
        y=(long*)calloc(ly,sizeof(long));
        for (k=0;k<ly;k++){
            int teller=0, p;
            for (p=ih;p<ih+lh;p++){
                //printf(" \n Og p er: %d", p);
                y[k]=y[k]+symData[p]*h[teller];
                //printf(" \n Og data[p] er: %f og h[lh-in+teller] er: %f ", (*inNode).data[p],h[lh-in+teller]);
                teller = teller+1;
            }
            y[k]/=10000;
            if (abs(y[k])>tmpMaxAbs){
                tmpMaxAbs=abs(y[k]);
            }
            //if(k>65&&k<100)printf(" %ld", (long)y[k]);

            ih=ih+2;
        }

        (*newNode).maxAbs=tmpMaxAbs;
        //printf("id: %d maxAbs %ld length %d\n",(*newNode).id,(*newNode).maxAbs,newNode->length);
        (*newNode).data=y;//the pointer to the child-node's data
        //printf("ldata: %d, ly: %d, (*newNode).id: %d\n", ldata,ly,(*newNode).id);
        //for (k=0;k<5;k++){printf(" %ld", y[k+40]);}printf(" \n");
        //if (newNode->id==17){ for (k=0;k<newNode->length;k++){printf(" %ld", y[k]);}printf(" \n");}

    }
    else{
        newNode->maxAbs=0;
        newNode->data=NULL;
        //printf("id: %d maxAbs %ld\n",(*newNode).id,(*newNode).maxAbs);
    }
    if ((*newNode).id<(1<<DEPTH)-1){
        biWavdec(newNode, (char)"l");
        biWavdec(newNode,(char)"r");
    }
    else{
        (*newNode).l=NULL;
        (*newNode).r=NULL;
    }
    free(symData);

}

/***************************************************************************
* Function for 1D WP analysis with decimation. The function recursively calls
* itself untill the desired depth is reach. A tree-structure which links every
* node together is created as well.
*---------------------------------------------------------------------------
* Input:   inNode   - pointer to a struct representing a node, with its
* 					  belonging data-sequence etc.
*          test    	- this value decides if the LP or HP filter is used
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-02-16
***************************************************************************/
void wavdec(NODE *inNode, char test){
    NODE *newNode=calloc(1,sizeof(NODE));
    short k,ldata=(*inNode).length, lh=10;
    short ly=(short)(lh+ldata-1)/2; //declaring length of data-array
    short ih=1;
    long *y; //pointer to array of filtered/decimated data
    long *h; //pointer to array of filter-coeffisents
    long *symData;
    unsigned long tmpMaxAbs=0;

    //checking if the new node is left or right child of parent
    if (test==(char)"l"){
        (*inNode).l=calloc(1,sizeof(NODE));
        newNode=(NODE*)(*inNode).l;//the link between child an parent
        (*newNode).id=2*(*inNode).id+1; //the child's node id
        //printf(" \n Det ble lavpass");
        h=L;
    }
    if (test==(char)"r"){
        (*inNode).r=calloc(1,sizeof(NODE));
        newNode=(NODE*)(*inNode).r;//the link between child an parent
        (*newNode).id=2*(*inNode).id+2; //the child's node id
        //printf(" \n Det ble hypass");
        h=H;
    }
    //newNode->p=inNode;//the link between child an parent
    (*newNode).length=ly; //the length of the child's data array
    //for (k=0;k<ly;k++){printf(" %f", data[k]);}
    y=(long*)calloc(ly,sizeof(long));
    //printf(" \n Lengde av y: %d", ly);

    //the following loop performs the actual filtering/decimating
    if(newNode->id!=(1<<DEPTH)-1)
    {
        symData=(long*)calloc(ldata+2*(lh-1),sizeof(long));
        for (k=0;k<lh-1;k++)
        {
            symData[lh-2-k]=inNode->data[k];
            symData[ldata+lh-1+k]=inNode->data[ldata-1-k];
        }
        for (k=0;k<ldata;k++)
        {
            symData[k+lh-1]=inNode->data[k];
        }
        y=(long*)calloc(ly,sizeof(long));
        for (k=0;k<ly;k++){
            int teller=0, p;
            for (p=ih;p<ih+lh;p++){
                //printf(" \n Og p er: %d", p);
                y[k]=y[k]+symData[p]*h[teller];
                //printf(" \n Og data[p] er: %f og h[lh-in+teller] er: %f ", (*inNode).data[p],h[lh-in+teller]);
                teller = teller+1;
            }
            y[k]/=10000;
            if (abs(y[k])>tmpMaxAbs){
                tmpMaxAbs=abs(y[k]);
            }
            //if(k>65&&k<100)printf(" %ld", (long)y[k]);

            ih=ih+2;
        }

        (*newNode).maxAbs=tmpMaxAbs;
        //printf("id: %d maxAbs %ld\n",(*newNode).id,(*newNode).maxAbs);
        (*newNode).data=y;//the pointer to the child-node's data
        //printf("ldata: %d, ly: %d, (*newNode).id: %d\n", ldata,ly,(*newNode).id);
        //for (k=0;k<5;k++){printf(" %ld", y[k]);}printf(" \n");
        //if (newNode->id==16){ for (k=0;k<newNode->length;k++){printf(" %ld", y[k]);}printf(" \n");}

    }
    else{
        newNode->maxAbs=0;
        newNode->data=NULL;
        //printf("id: %d maxAbs %ld\n",(*newNode).id,(*newNode).maxAbs);
    }
    if ((*newNode).id<(1<<DEPTH)-1){
        wavdec(newNode, (char)"l");
        wavdec(newNode,(char)"r");
    }
    else{
        (*newNode).l=NULL;
        (*newNode).r=NULL;
    }
    free(symData);

}

/***************************************************************************
* Function for calculating the mean square distortion and corresponding
* rate for each quantiziser for each node in the tree. The function recursively
* calls itself until all the nodes is reached. The quntizers are adjusted the
* dynamic range of the input data.
*---------------------------------------------------------------------------
* Input:   node   - pointer to a struct representing a node, with its belonging data
* 					sequence, distortion, rate etc.
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-02-16
***************************************************************************/
void initRD(NODE *node){

    unsigned short i,j,maxRate,N;
    long reprLevel, dataj,diff;
    long stepLength;

    //data = (float*)(*inNode).data;

    i=0;
    while (pow(2,i)<(2*(*node).maxAbs)){i++;}
    maxRate=i-1;
    if(maxRate>14)maxRate=14;
    (*node).nmbOfQ=maxRate;
    (*node).D_list=(unsigned long*)calloc(maxRate,sizeof(unsigned long));
    (*node).R_list=(unsigned char*)calloc(maxRate,sizeof(unsigned char));
    //printf("id: %d nmbOfQ %d\n",(*node).id,(*node).nmbOfQ);
    for (i=0;i<maxRate;i++){
        (*node).R_list[i]=(unsigned char)maxRate-i;
        N=(unsigned short)pow(2,(float)(*node).R_list[i]);
        //printf("id: %d N %d (*node).R_list[i] %d\n",(*node).id,N,(*node).R_list[i]);
        if(node->id==(1<<DEPTH)-1){
            (*node).R_list[i]=0;
            node->D_list[i]=0;
        }
        else if (node->id==0||node->id==1||node->id==3||node->id==7){//||(node->id<15&&node->id>8)||node->id>16 
            node->D_list[i]=1000000000;
        }
        else if (maxRate-i>1){
            stepLength=2*(*node).maxAbs/N;
			
			#ifdef USE_ESTIMATED_DISTORTION
	            if (stepLength*node->length<=(2000000000/stepLength)*12||stepLength<12){
					(*node).D_list[i]=stepLength*stepLength/12*node->length;
				}
				else{
					(*node).D_list[i]=1000000000;
				}
			#else
	            for (j=0;j<(*node).length;j++){
	                if(node->id==0){
	                    //printf("id: %d nmbOfQ %d\n",(*node).id,(*node).nmbOfQ);
	                }
	                dataj=(long)(*node).data[j];
	                if(node->id==1){
	                    //printf("id: %d dataj %ld\n",(*node).id,dataj);
	                }
	
	                if ((dataj<<1)<(-((*node).maxAbs<<1)+3*stepLength)){
	                    reprLevel=(long)((-(*node).maxAbs)+stepLength);
	                }
	                else if ((*node).data[j]>0){
	                    diff=dataj%(long)stepLength;
	                    if (((diff<<1)/stepLength)<1){
	                        reprLevel=dataj-diff;
	                    }
	                    else{
	                        reprLevel=dataj+(long)stepLength-diff;
	                    }
	                }
	                else{
	                    diff=abs(dataj%(long)stepLength);
	                    if (((diff<<1)/stepLength)<1){
	                        reprLevel=dataj+diff;
	                    }
	                    else{
	                        reprLevel=dataj-(long)stepLength+diff;
	                    }
	                }
	                (*node).D_list[i]+=(unsigned long)(reprLevel-(*node).data[j])*(reprLevel-(*node).data[j]);//accumulating distortion
	                //(*node).D_list[i]+=abs(((float)reprLevel-(*node).data[j]));
	            }
	    	#endif    
            //if(node->id<20&&node->id>7)printf("id: %d D_list[i] %ld R_list[i] %d i %d\n",(*node).id,(*node).D_list[i],(*node).R_list[i],i);

        }
        else{
            (*node).R_list[i]=0;
			//printf("InitRD else, id %d length %d %d\n",node->id, node->length,node->data[50]);
            if(node->maxAbs<30000){
                for (j=0;j<(*node).length;j++){
                    (*node).D_list[i]+=(*node).data[j]*(*node).data[j];//accumulating distortion
					//if (node->id<5&&j<36&&j>30)printf("N %d data %d acc %d\n", N,node->data[j],(*node).D_list[i]);

                    //(*node).D_list[i]+=(float)abs(((*node).data[j]));
                }
            }
            else{
                node->D_list[i]=1000000000;

            }
            //printf("id: %d D_list[i] %ld R_list[i] %d i %d\n",(*node).id,(*node).D_list[i],(*node).R_list[i],i);
        }

        if ((*node).id==pow(2,DEPTH)-1){//exploiting the fact that the lowpass information is not needed
            (*node).R_list[i]=0;
            (*node).D_list[i]=0;
            //printf("er i den hemmelige noden...id: %d %f\n",(*node).id,node->maxAbs);
        }

    }
    if ((*node).l!=NULL){initRD((NODE*)(*node).l);}
    if ((*node).r!=NULL){initRD((NODE*)(*node).r);}

}

/***************************************************************************
* Function for calculating the cost for a given lambda. The function also
* calculates the total rate for each node in bits. The function recursively calls
* itself so that all nodes is reached.
*---------------------------------------------------------------------------
* Input:   node   - pointer to a struct representing a node, with its belonging data
* 					sequence, distortion, rate etc.
* 		  lambda  - lambda
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-02-19
***************************************************************************/
void setCost(NODE *node, long *lambda){
    (*node).J=2000000000;

    short i;
    long tmpJ;
    for (i=0;i<(*node).nmbOfQ;i++){
        if ((*node).R_list[i]>0){
            tmpJ=(float)(*node).D_list[i]+(*lambda)*((*node).R_list[i]*((*node).length)+OVERHEAD);
        }
        else{
            tmpJ=(float)(*node).D_list[i];
        }
        if (tmpJ<(*node).J){
            (*node).J=tmpJ;
            (*node).R=(*node).R_list[i];
            (*node).D=(float)(*node).D_list[i];
            if((*node).R_list[i]>0){
                (*node).totR=(*node).R*(*node).length+OVERHEAD;
            }
            else{
                (*node).totR=0;
            }
        }
        /*if (node->id==25){
            printf("lambda = %f ID = %d tmpJ= %f rate= %f\n" ,*lambda, (*node).id,tmpJ,(*node).R_list[i]);
        }*/
    }
    //printf("lambda = %f ID = %d Cost= %f rate= %f\n" ,*lambda, (*node).id,(*node).J,(*node).R);
    if (((*node).l!=NULL)&&((*node).r!=NULL)){
        setCost((NODE*)(*node).l, lambda);
        setCost((NODE*)(*node).r, lambda);
    }

}

/***************************************************************************
* Function for finding the best subtree in a rate-distortion sense for a special lambda.
* Instead of brute-forcing all possible combinations of subtrees, dynamic programming is
* employed.
*---------------------------------------------------------------------------
* Input:   node   - pointer to a struct representing a node, with its belonging data
* 					sequence, distortion, rate etc.
* 			nmb	  - total number of nodes in the tree
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-02-21
***************************************************************************/
void findSubTree(NODE* node, short nmb){
    NODE *lchild, *rchild;
    lchild = (NODE*)(*node).l;
    rchild = (NODE*)(*node).r;
    //printf("ID= %d tall= %f\n",(*node).id,(float)nmb-(float)pow(2,DEPTH)-(float)pow(2,DEPTH-1));
    if ((*node).id<nmb-(1<<DEPTH)-(1<<(DEPTH-1))){
        findSubTree((NODE*)lchild,nmb);
        findSubTree((NODE*)rchild,nmb);
    }
    else{
        lchild->split=FALSE;
        rchild->split=FALSE;
    }

    //printf("costL= %f costR= %f costP= %f\n",(*lchild).J,(*rchild).J,(*node).J);
    if ((((*lchild).J+(*rchild).J)<(*node).J)&&((*lchild).R>0||(*rchild).R>0)){

        (*node).split=TRUE;
        (*node).totR=(*rchild).totR+(*lchild).totR;
        (*node).J=(*rchild).J+(*lchild).J;
        (*node).D=(*rchild).D+(*lchild).D;
    }
    else{
        (*node).split=FALSE;
        //printf("No split: PID= %d LID: %d RID: %d\n",(*node).id,(*lchild).id,(*rchild).id);
    }
    //printf("ID %d split %d totR %f D %f\n",(*node).id,(*node).split,(*node).R,(*node).D);
}

/***************************************************************************
* Function for finding the best subtree in a rate-distortion sense. Instead of
* brute-forcing all possible combinations of subtrees, dynamic programming is
* employed.
*---------------------------------------------------------------------------
* Input:   root   - pointer to the root-node, with its belonging data
* 					sequence, distortion, rate etc.
* 			nmb	  - total number of nodes in the tree
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-05-02
***************************************************************************/
void findBestSubTree(NODE* root, short nmb){
	long RUL,RLL,DUL,DLL,*lambda,lambdaUL=100000, lambdaLL=0;
    short testLambda=FALSE,i=0;
    lambda=(long*)calloc(1,sizeof(long));
     
	while((testLambda==FALSE)&&(i<15)){
        //finding the initial values of the rate/distortion for the upper/lower limit of lambda
        setCost(root,&lambdaUL);
        findSubTree(root, nmb);
        RUL=(*root).totR;
        DUL=(*root).D;
        //printf("Upper limit RUL: %ld D: %ld totR: %ld lambdaUL %ld\n",RUL,(*root).D,(*root).totR,lambdaUL);
        setCost(root,&lambdaLL);
        findSubTree(root, nmb);
        RLL=(*root).totR;
        DLL=(*root).D;
        //printf("Lower limit RLL: %ld D: %ld totR: %ld lambdaLL %ld\n",RLL,(*root).D,(*root).totR,lambdaLL);
        *lambda=((DUL-DLL)/(RLL-RUL))+1;
        //*lambda=(lambdaLL+lambdaUL)/2.0;
        //*lambda=88000;
        //finding the best cost/quantiziser/rate for this lambda for every node
        setCost(root,lambda);
        //finding the best basis in a R-D sense
        findSubTree(root, nmb);
        if (RUL==(*root).totR){
            testLambda=TRUE;
        }
        else if ((*root).totR>RBUDGET){
            lambdaLL=*lambda;
        }
        else {lambdaUL=*lambda;}
        //printf("lambdaUL: %ld lambdaLL: %ld i: %d lambda: %ld totR: %ld J: %ld\n",lambdaUL,lambdaLL,i,*lambda,(*root).totR,(*root).J);
        i++;
    }
    //printf("totR: %ld\n",(*root).totR);
}


/***************************************************************************
* Function for creating the structs representing the leaf-nodes whos data is
* going to be quantizised. The leaf structs in addition represents all the
* relevant metadata for decoding.
*---------------------------------------------------------------------------
* Input:   node   - pointer to a struct representing a node, with its belonging data
* 					sequence, distortion, rate etc.
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-02-23
***************************************************************************/
void createLeaves(NODE* node, FILE *fOut){
    if ((*node).split==TRUE){
        createLeaves((NODE*)(*node).l,fOut);
        createLeaves((NODE*)(*node).r,fOut);
    }
    else{
        LEAF * leaf = calloc(1,sizeof(LEAF));
        (*leaf).id=(unsigned char)(*node).id;
        (*leaf).R=(unsigned char)(*node).R;
        (*leaf).length=(unsigned short)(*node).length;
        (*leaf).maxAbs=(unsigned short)(*node).maxAbs;
        if ((*leaf).R>0){
        	unifQuantDead((NODE*)node, (LEAF*)leaf);
	    	
	    	rle(leaf);

            writeLeaf((LEAF*)leaf,fOut);

        }
        //printf("\n ID %d maxAbs %d",leaf->id,leaf->maxAbs);
    }
}



/***************************************************************************
* Function for uniform quantization with deadzone. The indicies of the quantizised data are
* put in the leaf's data-array.
*---------------------------------------------------------------------------
* Input:   node   - pointer to a struct representing a node, with its belonging data
* 					sequence, distortion, rate etc.
* 		   leaf   -	pointer to a struct representing a leaf in the optimal subtree
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-04-30
***************************************************************************/
void unifQuantDead(NODE* node, LEAF * leaf){
    long stepLengthOver,stepLengthUnder,stepLength,tmp,tmp2,tmp3;
    short N,i,zeroRepresentation;
    if(DEADZONE_NUM/DEADZONE_DEN>2){
    	if (leaf->R==2) {
    		leaf->R=3;
    		node->R=3;
    	}
    }
    N=(short)pow(2,node->R);
    zeroRepresentation=(N>>1)-1;
    stepLength = 2*node->maxAbs/N;
    tmp3=((stepLength*DEADZONE_NUM)>>1)/DEADZONE_DEN;//be shure that this calculation only is performed once...
    stepLengthOver=(node->maxAbs-tmp3)/(N>>1);
    stepLengthUnder=(node->maxAbs-tmp3)/((N>>1)-1);
    
    leaf->data=(unsigned short*)calloc((*leaf).length,sizeof(unsigned short));
    //printf("\nID %d R %d maxA %d length %d tmp3 %ld\n",leaf->id,leaf->R,leaf->maxAbs,leaf->length,tmp3);
    for (i=0;i<(*leaf).length;i++){
    	if(abs(node->data[i])<tmp3){
    		leaf->data[i]=zeroRepresentation;
    	}
        else if(node->data[i]>0){
            tmp=node->data[i]-tmp3;
            leaf->data[i]=(unsigned short)zeroRepresentation+(tmp/stepLengthOver)+1;
			if (leaf->data[i]>=N){
				leaf->data[i]=N-1;
			}
            
            //if (leaf->id==25) printf("shift %d leaf->R %d\n",(2<<(short)(leaf->R-2))-1,leaf->R);
        }
        else{
        	
            tmp=node->data[i]+tmp3;
            tmp2=(short)zeroRepresentation+(tmp/stepLengthUnder)-1;
            //printf(" %d %f %f ",(short)(tmp/stepLength),tmp, tmp2);
            if(tmp2>0){
                leaf->data[i]=(unsigned short)tmp2;
            }
            else{
                leaf->data[i]=0;
            }
            

        }
        //printf(" %d",leaf->data[i]);
        //if(node->id==17){printf(" %ld",node->data[i]);}

    }
    if (kr==93||kr==94||kr==95){
    	printf(" %d ID %d R %d maxA %d var %ld\n",kr,leaf->id,leaf->R,leaf->maxAbs,node->variance);
    }
    
    //printf("\n ID %d R %d maxA %d length %d nmbZeros %d\n",leaf->id,leaf->R,leaf->maxAbs,leaf->length,leaf->nmbZeros);
}

/***************************************************************************
* Function for run length encoding
*---------------------------------------------------------------------------
* Input:   leaf   -	pointer to a struct representing a leaf in the optimal subtree
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-03-19
***************************************************************************/
void rle(LEAF* leaf){
    short rl=-1;
    unsigned short *y,past,yi=1,i,tmpR;
    y=(unsigned short*)calloc(2*leaf->length,sizeof(unsigned short));
    y[0]=leaf->data[0];
    past = leaf->data[0];
    tmpR=leaf->R;
    if (leaf->R<4){
        tmpR=4;
    }
    for (i=1;i<leaf->length;i++){
        if (past==leaf->data[i]){
            rl++;
            if (rl>=(2<<(tmpR-1))-1){
                y[yi]=past;
                y[yi+1]=rl;
                yi+=2;
                rl=-1;
                past=1<<14;
            }
        }
        else if ((past!=leaf->data[i])&&rl>-1){
            y[yi]=past;
            y[yi+1]=rl;
            y[yi+2]=leaf->data[i];
            yi+=3;
            rl=-1;
            past=leaf->data[i];
        }
        else{
            y[yi]=leaf->data[i];
            yi++;
            past=leaf->data[i];
        }

    }
    if (rl>-1){
        y[yi]=past;
        y[yi+1]=rl;
        yi+=2;
    }
    leaf->lengthRl=yi;
    leaf->rlData=y;
    if ((leaf->length*leaf->R)>(leaf->lengthRl*tmpR+10)){
        leaf->isRLenc=TRUE;
        //printf("Id %d rle\n ",leaf->id);
    }
    else{
        leaf->isRLenc=FALSE;
        //printf("Id %d no rle\n ",leaf->id);
    }
    //printf("\nAfter rle: id %dlengthRl %d length %d\n ",leaf->id,leaf->lengthRl,leaf->length);
    for (i=0;i<leaf->lengthRl;i++){
        //printf(" %d ",leaf->rlData[i]);
    }
}

/***************************************************************************
* Function for writing a leaf to an outputfile
*---------------------------------------------------------------------------
* Input:   leaf   -	pointer to a struct representing a leaf in the optimal subtree
* 		   fOut	  - pointer to file which the metadata is written to
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-02-24
***************************************************************************/
void writeLeaf(LEAF* leaf,FILE* fOut){
    unsigned char bitbuffer,j,y1,y2,tmpR;
    unsigned short mask,x,i,*y,length;
    /********************************************************************************
    * Writing header (a total of 26 bits for each leaf):
    * -maxAbs is the absolute value of the biggest value in the dataset, 16 bits
    * -id is the unique id of the node so that it is possible to reconstruct the signal
    * 	at the decoder, represented by 6 bits
    * -Rate is the number og bits each sample is represented with, this field is
    * 	represented with 4 bits
    * ********************************************************************************/
    //write maxAbs (16 bits)
    y1=0;y2=0;
    x=leaf->maxAbs;
    y2=x;
    x>>=8;
    y1=x;
    fputc(y1,fOut);
    fputc(y2,fOut);
    //write id (6 bits)
    mask=1<<5;
    bitbuffer=0;
    x=(unsigned short)leaf->id;
    y1=0;
    for (i=0;i<6;i++){
        y1=(((x&mask)>>5)|y1);
        bitbuffer++;
        y1<<=1;
        x<<=1;
    }
    //write rate(4 bits)
    mask=1<<3;
    x=leaf->R;
    for (i=0;i<4;i++){
        y1=(((x&mask)>>3)|y1);
        bitbuffer++;
        if (bitbuffer==8){
            fputc(y1,fOut);
            y1=0;
            bitbuffer=0;
        }
        y1<<=1;
        x<<=1;
    }

    //write isRLenc
    mask=1;
    x=leaf->isRLenc;
    y1=x|y1;
    bitbuffer++;
    y1<<=1;
    x<<=1;
    //write length of rle data if used
    if(leaf->isRLenc==TRUE){
        //if(leaf->id==16)printf("2: R %d y1 %d x %d bitbuffer %d\n",leaf->R,y1,x,bitbuffer);
		
        mask=1<<9;
        x=leaf->lengthRl;
        for (i=0;i<10;i++){
            y1=(((x&mask)>>9)|y1);
            bitbuffer++;
            if (bitbuffer==8){
                fputc(y1,fOut);
                y1=0;
                bitbuffer=0;
            }
            y1<<=1;
            x<<=1;
        }
        y=leaf->rlData;
        length=leaf->lengthRl;
    }
	else{
        y=leaf->data;
        length=leaf->length;
    }
    /***********************************************************
    * Writing quantizised data to output file
    * *********************************************************/
    if(leaf->R<4)
    {
        tmpR=4;
    }
    else
    {
        tmpR=leaf->R;
    }
    mask=1<<(tmpR-1);

    for (i=0;i<length;i++)
    {
        x=(unsigned short)y[i];
        //if(leaf->id==17 && i<10)printf("\nx %d\n",x);

        for (j=0;j<tmpR;j++){
            //if(leaf->id==17 && i<10)printf("((x&mask)>>(tmpR-1)) %d x %d\n",((x&mask)>>(tmpR-1)),x);
            y1=(((x&mask)>>(tmpR-1))|y1);
            //if(leaf->id==17 && i<10)printf("R %d y1 %x x %d bitbuffer %d y %d\n",leaf->R,y1,x,bitbuffer,y[i]);
            bitbuffer++;
            if (bitbuffer==8){
                //if(leaf->id==17)printf(" %d",y1);
                fputc(y1,fOut);
                y1=0;
                bitbuffer=0;
            }
            y1<<=1;
            x<<=1;
        }
    }

    //writing out the bits stored in y1
    if (bitbuffer>0){
        y1<<=(8-bitbuffer-1);
        fputc(y1,fOut);
    }
}



/***************************************************************************
* Function for creating the structs representing the leaf-nodes whos data is
* going to be quantizised. The leaf structs in addition represents all the
* relevant metadata for decoding.
*---------------------------------------------------------------------------
* Input:   nodes   - pointer to pointers of structs representing a node, with its belonging data
* 					sequence, distortion, rate etc.
* 		   fOut	   - pointer to output stream	
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-04-29
***************************************************************************/
void writeLeaves(NODE** nodes, FILE *fOut){
    short i;
    NODE* node;
    for (i=1;i<(1<<DEPTH);i++){
    	node=(NODE*)nodes[i];
	    LEAF * leaf = calloc(1,sizeof(LEAF));
	    (*leaf).id=(unsigned char)(*node).id;
	    (*leaf).R=(unsigned char)(*node).R;
	    if(leaf->R==1){
	    	leaf->R=2;//want minimum four bits
	    	node->R=2;
	    }
	    (*leaf).length=(unsigned short)(*node).length;
	    (*leaf).maxAbs=(unsigned short)(*node).maxAbs;
	    if ((*leaf).R>0){
	    	unifQuantDead((NODE*)node, (LEAF*)leaf);
	    	rle(leaf);
	        writeLeaf((LEAF*)leaf,fOut);
	
	    }
	    //printf("\n ID %d maxAbs %d",leaf->id,leaf->maxAbs);
    }
}

/***************************************************************************
* Function for 1D WP analysis with decimationfor biorthogonal wavelets. The function recursively calls
* itself untill the desired depth is reach. A tree-structure which links every
* node together is created as well.
*---------------------------------------------------------------------------
* Input:   inNode   - pointer to a struct representing a node, with its
* 					  belonging data-sequence etc.
*          test    	- this value decides if the LP or HP filter is used
* 		   leafNodes - pointer to pointers of leafNodes
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-04-26
***************************************************************************/
void biWavdecRE(NODE *inNode, char test,NODE **leafNodes){
				
	NODE *newNode=calloc(1,sizeof(NODE));
    short k,ldata=(*inNode).length,lh;
    short ly; //declaring length of data-array
    short ih;
    long *y; //pointer to array of filtered/decimated data
    long *h; //pointer to array of filter-coeffisents
    long *symData;
    unsigned long tmpMaxAbs=0;

    //checking if the new node is left or right child of parent
    if (test==(char)"l"){
        (*inNode).l=calloc(1,sizeof(NODE));
        newNode=(NODE*)(*inNode).l;//the link between child an parent
        (*newNode).id=2*(*inNode).id+1; //the child's node id
        //printf(" \n Det ble lavpass");
        h=L;
        lh=BI_LP_FILTERLENGTH;
        ih=0;
        ly=(short)(ldata+1)/2;
    }
    if (test==(char)"r"){
        (*inNode).r=calloc(1,sizeof(NODE));
        newNode=(NODE*)(*inNode).r;//the link between child an parent
        (*newNode).id=2*(*inNode).id+2; //the child's node id
        //printf(" \n Det ble hypass");
        h=H;
        lh=BI_HP_FILTERLENGTH;
        ih=2;
        ly=ldata-(short)(ldata+1)/2;
    }
    //newNode->p=inNode;//the link between child an parent
    (*newNode).length=ly; //the length of the child's data array
    //for (k=0;k<ly;k++){printf(" %f", data[k]);}
    y=(long*)calloc(ly,sizeof(long));
    //printf(" \n Lengde av y: %d", ly);

    //the following loop performs the actual filtering/decimating
    if(newNode->id!=(1<<DEPTH)-1)
    {
        symData=(long*)calloc(ldata+8,sizeof(long));
	    for (k=0;k<4;k++)
	    {
	        symData[3-k]=inNode->data[k+1];
	        symData[ldata+4+k]=inNode->data[ldata-2-k];
	    }
	    for (k=0;k<ldata;k++)
	    {
	        symData[k+4]=inNode->data[k];
	    }
        y=(long*)calloc(ly,sizeof(long));
        for (k=0;k<ly;k++){
            int teller=0, p;
            for (p=ih;p<ih+lh;p++){
                //printf(" \n Og p er: %d", p);
                y[k]=y[k]+symData[p]*h[teller];
                //printf(" \n Og data[p] er: %f og h[lh-in+teller] er: %f ", (*inNode).data[p],h[lh-in+teller]);
                teller = teller+1;
            }
            y[k]/=10000;
            if (abs(y[k])>tmpMaxAbs){
                tmpMaxAbs=abs(y[k]);
            }
            //if(k>65&&k<100)printf(" %ld", (long)y[k]);

            ih=ih+2;
        }

        (*newNode).maxAbs=tmpMaxAbs;
        //printf("id: %d maxAbs %ld length %d\n",(*newNode).id,(*newNode).maxAbs,newNode->length);
        (*newNode).data=y;//the pointer to the child-node's data
        //printf("ldata: %d, ly: %d, (*newNode).id: %d\n", ldata,ly,(*newNode).id);
        //for (k=0;k<5;k++){printf(" %ld", y[k+40]);}printf(" \n");
        //if (newNode->id==17){ for (k=0;k<newNode->length;k++){printf(" %ld", y[k]);}printf(" \n");}

    }
	else{
		newNode->maxAbs=0;
		newNode->data=NULL;
		//printf("id: %d maxAbs %ld\n",(*newNode).id,(*newNode).maxAbs);
	}
	if ((*newNode).id<(1<<DEPTH)-1){
		biWavdecRE(newNode, (char)"l",leafNodes);
		biWavdecRE(newNode,(char)"r",leafNodes);
		
	}
	else{
		leafNodes[(*newNode).id-(1<<DEPTH)+1]=newNode;
		(*newNode).l=NULL;
		(*newNode).r=NULL;
		newNode->R=0;
		if(newNode->id>(1<<DEPTH)-1){
			newNode->variance=0;
			for (k=0;k<newNode->length;k++){
				newNode->variance+=(newNode->data[k]>>2)*(newNode->data[k]>>2);
			}
			//printf("id: %d variance %ld\n",(*newNode).id,(*newNode).variance);
		}
	}
	free(symData);
}

/***************************************************************************
* Function for 1D WP analysis with decimation. The function recursively calls
* itself untill the desired depth is reach. A tree-structure which links every
* node together is created as well.
*---------------------------------------------------------------------------
* Input:   inNode   - pointer to a struct representing a node, with its
* 					  belonging data-sequence etc.
*          test    	- this value decides if the LP or HP filter is used
* 		   leafNodes - pointer to pointers of leafNodes
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-02-16
***************************************************************************/
void wavdecRE(NODE *inNode, char test,NODE **leafNodes){
				
	NODE *newNode=calloc(1,sizeof(NODE));
	short k,ldata=(*inNode).length, lh=10;
	short ly=(short)(lh+ldata-1)/2; //declaring length of data-array
	short ih=1;
	long *y; //pointer to array of filtered/decimated data
	long *h; //pointer to array of filter-coeffisents
	long *symData;
	unsigned long tmpMaxAbs=0;
			
	//checking if the new node is left or right child of parent
	if (test==(char)"l"){
		(*inNode).l=calloc(1,sizeof(NODE));
		newNode=(NODE*)(*inNode).l;//the link between child an parent
		(*newNode).id=2*(*inNode).id+1; //the child's node id
		//printf(" \n Det ble lavpass");
		h=L;
	}
	if (test==(char)"r"){
		(*inNode).r=calloc(1,sizeof(NODE));
		newNode=(NODE*)(*inNode).r;//the link between child an parent
		(*newNode).id=2*(*inNode).id+2; //the child's node id
		//printf(" \n Det ble hypass");
		h=H;
	}
	//newNode->p=inNode;//the link between child an parent
	(*newNode).length=ly; //the length of the child's data array
	//for (k=0;k<ly;k++){printf(" %f", data[k]);}
	y=(long*)calloc(ly,sizeof(long));
	//printf(" \n Lengde av y: %d", ly);
	
	//the following loop performs the actual filtering/decimating
	if(newNode->id!=(1<<DEPTH)-1){
		symData=(long*)calloc(ldata+2*(lh-1),sizeof(long));
		for (k=0;k<lh-1;k++)
		{
			symData[lh-2-k]=inNode->data[k];
			symData[ldata+lh-1+k]=inNode->data[ldata-1-k];
		}
		for (k=0;k<ldata;k++)
		{
			symData[k+lh-1]=inNode->data[k];
		}		
		y=(long*)calloc(ly,sizeof(long));
		for (k=0;k<ly;k++){
		    int teller=0, p;
		    for (p=ih;p<ih+lh;p++){
		    	//printf(" \n Og p er: %d", p);
		        y[k]=y[k]+symData[p]*h[teller];
		        //printf(" \n Og data[p] er: %f og h[lh-in+teller] er: %f ", (*inNode).data[p],h[lh-in+teller]);
		        teller = teller+1;
		    }
		    y[k]/=10000;
		    if (abs(y[k])>tmpMaxAbs){
		    	tmpMaxAbs=abs(y[k]);
		    }
		    //if(k>65&&k<100)printf(" %ld", (long)y[k]);
		    
		    ih=ih+2;
		}
		
		(*newNode).maxAbs=tmpMaxAbs;
		//printf("id: %d maxAbs %ld\n",(*newNode).id,(*newNode).maxAbs);
		(*newNode).data=y;//the pointer to the child-node's data
		//printf(" \n ldata: %d, ly: %d, (*newNode).id: %d\n", ldata,ly,(*newNode).id);
		//for (k=0;k<5;k++){printf(" %ld", y[k]);}printf(" \n");
		//if (newNode->id==16){ for (k=0;k<newNode->length;k++){printf(" %ld", y[k]);}printf(" \n");}
		
	}
	else{
		newNode->maxAbs=0;
		newNode->data=NULL;
		//printf("id: %d maxAbs %ld\n",(*newNode).id,(*newNode).maxAbs);
	}
	if ((*newNode).id<(1<<DEPTH)-1){
		wavdecRE(newNode, (char)"l",leafNodes);
		wavdecRE(newNode,(char)"r",leafNodes);
	}
	else{
		leafNodes[(*newNode).id-(1<<DEPTH)+1]=newNode;
		(*newNode).l=NULL;
		(*newNode).r=NULL;
		newNode->R=0;
		if(newNode->id>(1<<DEPTH)-1){
			newNode->variance=0;
			for (k=0;k<newNode->length;k++){
				newNode->variance+=(newNode->data[k]>>2)*(newNode->data[k]>>2);
			}
			//printf("id: %d variance %ld\n",(*newNode).id,(*newNode).variance);
		}
	}
	free(symData);
}

/***************************************************************************
* Function for allocating bits. Greedy algorithm, gives bits to subband with highest variance
* and then dividing its variance with a certain factor
*---------------------------------------------------------------------------
* Input:   nodes   -	pointer to pointers of structs representing a node, with its belonging data
* 					sequence, distortion, rate etc.
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-04-29
***************************************************************************/
void allocateBits(NODE** nodes){
	short test = TRUE,i,rate,index,numerator;
	long tmpMax;
	rate=0;
	numerator=0;
	while (test){
		//finding the highest variance
		tmpMax=0;
		index=-1;
		
		for (i=1;i<(1<<DEPTH);i++){
			if (nodes[i]->variance>tmpMax){
				index=i;
				tmpMax=nodes[i]->variance;
			}
		}
		nodes[index]->R++;
		nodes[index]->variance=(unsigned long)nodes[index]->variance/(NUM_VAR/DEN_VAR);
		//printf("okeydooky id %d R %d varians %ld rate %d\n",nodes[index]->id,nodes[index]->R,nodes[index]->variance,rate);
		if ((1<<(nodes[index]->R+2)>nodes[index]->maxAbs)||(nodes[index]->R==15))
		{
			nodes[index]->variance=0;
			numerator++;
				if(numerator==(1<<DEPTH)-1){
					test=FALSE;	
				}
		
		}
		rate+=nodes[index]->length;
		if(rate>=RBUDGET){
			test=FALSE;
		}
	}
	
}

/***************************************************************************
* Function for writing unsigned short of original signal after cut              
*---------------------------------------------------------------------------
* Input:   us   -    the unsigned short to write
* 			fOut		-	output stream
*---------------------------------------------------------------------------
* Tarjei Rommetveit        2006-04-21 
***************************************************************************/
void writeUS(unsigned short us, FILE* fOut){
	
	unsigned char y1,y2;
	y1=0;y2=0;
    y2=us;
    us>>=8;
    y1=us;
    fputc(y1,fOut);
    fputc(y2,fOut);
}

void getTree(NODE* node){
	if ((*node).split==TRUE){
        getTree((NODE*)(*node).l);
        getTree((NODE*)(*node).r);
        printf("IF id %d R %ld\n", node->id, node->R);
    }
    else{
    	loev[node->id]=node->id;
    	rater[node->id]=node->R;
    	printf("ELSE id %d R %ld\n", node->id, node->R);
    }
    printf(" id %d R %ld\n", node->id, node->R);
}

void setTree(NODE* node){
	if ((*node).l!=NULL){
        setTree((NODE*)(*node).l);
        setTree((NODE*)(*node).r);
    }
    if (loev[node->id]!=0){
    	node->split=FALSE;
    	node->R=rater[node->id];
    }
    else{
    	if ((*node).id<15){
	    	node->split=TRUE;
	    	node->R=0;
    	}
    	else{
    		node->split=FALSE;
    		node->R=rater[node->id];
    	}
    }	
	
}
	

