function [C, L]=biwav_dec(x, wavname, depth)
%BIWAV_DEC Multi-level discrete 1-D biorthogonal wavelet decomposition.
%   
%   [C,L]=biwav_dec(x,'wavname',N) computes the N-level biorthogonal wavelet decomposition
%   of x with the wavelet 'wavname'. Symmetric extension is employed to
%   avoid coefficient expansion.
%   Currently supported wavelets: '4_4'. This corresponds to the 7/9-tap JPEG2000 filter.
%   
%   C is a vector containing the wavelet coeffesients: 
%   C=[AC(N) DC(N) DC(N-1)...DC(1)].
%   L is a vector containing the number of coeffisents for each level. The
%   last element expresses the number of elements in the original signal.

lx=length(x);
s=single(x);
if(size(x,1)>1)
    x=x';
end
switch wavname
    case '4_4'
        HD= [-0.0645388826286971 0.0406894176091641 0.418092273221617 -0.788485616405583 0.418092273221617 0.0406894176091641 -0.0645388826286971];
        LD= [0.037828455507264 -0.0238494650195568 -0.110624404418437 0.377402855612831 0.852698679008894 0.377402855612831 -0.110624404418437 -0.0238494650195568 0.037828455507264];
end
HD=double(single(fliplr(HD)));
LD=double(single(fliplr(LD)));
L=[];
C=[];

for i=1:depth
    lxold=length(x);
    for j=1:4
        x1(5-j)=x(j+1);
        x2(j)=x(lxold-j);
    end
    x=[x1 x x2];
    lac=floor((lxold+1)/2);
    ldc=lxold-lac;
    dc=double(single(wavconvD(HD,x,ldc)));
    ac=double(single(wavconvD(LD,x,lac)));
    
    
    if size(ac,1)>1
        ac=ac';
        dc=dc';
    end
    L=[length(dc) L];
    C=double(single([dc C]));
    x=ac;
end
C=[ac C];
L=[length(ac) L lx];
%idwt(C(1:L(1)),C(L(1)+1:L(1)+L(2)),'db3')-[1 2 3 4 5 6 7 8 9]

%-----------------------------------------------------%
% Internal Function(s)
%-----------------------------------------------------%
function y=wavconvD(h,xn,ly)
lh=length(h);
xni=1;
if lh==7
    hi=3;
else
    hi=1;
end
y=zeros(1,ly);
for k=1:ly
    teller=1;
    for p=hi:(hi+lh-1)
        y(k)=double(single(y(k)+xn(p)*h(teller)));
        teller = teller+1;
    end
    hi=hi+2;
end
        
    


