function x=biwav_rec(C,L,wname)
%BIWAV_REC Multi-level discrete 1-D wavelet reconstruction using biorthogonal wavelets.
%   
%   x=biwav_rec(C,L,wname) returns the reconstructed signal based on the vectors L and C
%   computed with wav_dec_sym. 
%   Currently supported wavelets: '4_4'. This corresponds to the 7/9-tap
%   JPEG2000 filter.
%   
%   C is a vector containing the wavelet coeffesients: 
%   C=[AC(N) DC(N) DC(N-1)...DC(1)].
%   L is a vector containing the number of coeffisents for each level. The
%   last element expresses the number of elements in the original signal.
%
%   See biwav_dec.
switch wname
    case '4_4'
       HR= [0 -0.037828455507264 -0.0238494650195568 0.110624404418437 0.377402855612831 -0.852698679008894 0.377402855612831 0.110624404418437 -0.0238494650195568 -0.037828455507264];
       LR= [0 -0.0645388826286971 -0.0406894176091641 0.418092273221617 0.788485616405583 0.418092273221617 -0.0406894176091641 -0.0645388826286971 0 0];
end

depth=length(L)-2;
Ls=L(1:end-2);
for i=2:length(Ls)
    Ls(i)=Ls(i)+Ls(i-1);
end
Ls=[0 Ls]+1;
ac=C(Ls(1):Ls(2)-1);
for i=1:depth
    if (i+2<=length(Ls))
        dc=C(Ls(i+1):(Ls(i+2)-1));
    else
        dc=C(Ls(i+1):end);
    end
    lac=length(ac);
    ldc=length(dc);
    if (rem(lac+ldc,2)==0)
        for j=1:3
            dc1(4-j)=dc(j); %dc (2,1) if even parent
            dc2(j)=dc(length(dc)-j);
            ac1(4-j)=ac(j+1); %ac (1,2) if even parent
            ac2(j)=ac(length(ac)-j+1);
        end
    else
        for j=1:3
            dc1(4-j)=dc(j); %dc (2,2) if odd parent
            dc2(j)=dc(length(dc)-j+1);
            ac1(4-j)=ac(j+1); %ac (1,1) if odd parent
            ac2(j)=ac(length(ac)-j);
        end
    end
        
    dc=[dc1 dc dc2];
        
    ac=[ac1(2:end) ac ac2];
    aci=inter(ac);
    dci=inter(dc);
    
    dc=wavconvR(HR,dci,lac+ldc);
    ac=wavconvR(LR,aci,lac+ldc);
    ac=ac+dc;
    
end
x=ac;



%-----------------------------------------------------%
% Internal Function(s)
%-----------------------------------------------------%
function y=wavconvR(h,xn,lrec)
lh=length(h);
hi=1;
y=zeros(1,lrec);
ly=lrec;

for i=1:ly
    teller=1;
    for j=hi:hi+lh-1
        y(i)=y(i)+xn(j)*h(teller);
        teller = teller+1;
    end
    hi=hi+1;
end

function y=inter(x)
lx=length(x);
y=zeros(1,lx*2);
for i=1:lx
    y(2*i-1)=x(i);
end