function x=wav_rec(C,L,wname)
%WAV_REC Multi-level discrete 1-D wavelet reconstruction.
%   
%   x=wav_rec(C,L,wname) returns the reconstructed signal based on the vectors L and C
%   computed with wav_dec_sym. 
%   Currently supported wavelets: 'db3', 'db5'.
%   
%   C is a vector containing the wavelet coeffesients: 
%   C=[AC(N) DC(N) DC(N-1)...DC(1)].
%   L is a vector containing the number of coeffisents for each level. The
%   last element expresses the number of elements in the original signal.
%
%   See wav_dec_sym.
switch wname
    case 'db3'
       HR=[0.0352262918821007 0.0854412738822415 -0.135011020010391 -0.459877502119331 0.806891509313339 -0.332670552950957];
       LR=[0.332670552950957 0.806891509313339 0.459877502119331 -0.135011020010391 -0.0854412738822415 0.0352262918821007];
       n=3;
    case 'db5'
       HR=[0.00333572528500155 0.0125807519990155 -0.00624149021301171 -0.0775714938400651 -0.0322448695850295 0.24229488706619 0.138428145901103 -0.724308528438574 0.603829269797473 -0.160102397974125];
       LR=[0.160102397974125 0.603829269797473 0.724308528438574 0.138428145901103 -0.24229488706619 -0.0322448695850295 0.0775714938400651 -0.00624149021301171 -0.0125807519990155 0.00333572528500155];
       n=5; 
    
end
HR=double(single(HR));
LR=double(single(LR));
C=double(single(C));
depth=length(L)-2;
Ls=L(1:end-2);
for i=2:length(Ls)
    Ls(i)=Ls(i)+Ls(i-1);
end
Ls=[0 Ls]+1;
ac=C(Ls(1):Ls(2)-1);
for i=1:depth
    if (i+2<=length(Ls))
        dc=C(Ls(i+1):(Ls(i+2)-1));
    else
        dc=C(Ls(i+1):end);
    end
    aci=double(single(inter(ac)));
    dci=double(single(inter(dc)));
    if n>0
        dc=double(single(wavconvR(HR,dci,n,L(2+i))));
        ac=double(single(wavconvR(LR,aci,n,L(2+i))));
    else
        dc=double(single(wavconvR(HR,dci,3.5,L(2+i))));
        ac=double(single(wavconvR(LR,aci,4.5,L(2+i))));
    end
    ac=double(single(ac+dc));
end
x=ac;



%-----------------------------------------------------%
% Internal Function(s)
%-----------------------------------------------------%
function y=wavconvR(h,xn,worder,lrec)
in=2*(worder-1);
h=fliplr(h);
lx=length(xn);
lh=length(h);
startxn=1;
hi=2-lh+in;
y=zeros(1,lrec);
ly=lrec;
for i=1:ly
    teller=0;
    for j=max(startxn,hi):min(hi+lh-1,lx)
        y(i)=double(single(y(i)+xn(j)*h(lh-in+teller)));
        teller = teller+1;
    end
    if (in+1<lh)
        in=in+1;
    end
    hi=hi+1;
end

function y=inter(x)
lx=length(x);
y=zeros(1,lx*2);
for i=1:lx
    y(2*i-1)=x(i);
end